import os
import tkinter as tk
import tkinter.ttk as ttk
import webbrowser

from PIL import Image, ImageTk

from ..classes.profile import ProfileHandler


class show_info:
    def __init__(self, parent, lang, mode):

        author = {
            "de": "TopasGraphSim\n\nAutor: Sebastian Schäfer",
            "en": "TopasGraphSim\n\nAuthor: Sebastian Schäfer",
        }
        version = {"de": "Version: 20.0.2\n ", "en": "Version: 20.0.2\n "}

        if mode == True:
            pic = "light"
        else:
            pic = "dark"

        window = tk.Toplevel()
        window.title("")
        window.resizable(False, False)
        window.geometry(f"180x240+{parent.winfo_rootx()}+{parent.winfo_rooty()}")
        window.wm_attributes("-toolwindow", True)
        im = Image.open(
            os.path.join(os.path.dirname(os.path.realpath(__file__)), "icon.png")
        ).resize((64, 64))
        ph = ImageTk.PhotoImage(im)
        imagelabel = tk.Label(window, image=ph,)
        imagelabel.image = ph
        authorlabel = tk.Label(window, text=author[lang])
        versionlabel = tk.Label(window, text=version[lang])
        icon = {True: "✅", False: "❌"}
        DnDlabel = tk.Label(
            window,
            text=f"Drag and Drop: {icon[ProfileHandler().get_attribute('draganddrop')]}",
        )
        ghimage = Image.open(
            os.path.join(os.path.dirname(os.path.realpath(__file__)), f"gh_{pic}.png")
        )
        self.ghimage = ImageTk.PhotoImage(ghimage)
        button = ttk.Button(window, image=self.ghimage, command=self.open_github)
        imagelabel.pack()
        authorlabel.pack()
        DnDlabel.pack()
        versionlabel.pack()
        button.pack()

    def open_github(self):
        webbrowser.open("https://github.com/sebasj13/topasgraphsim")



import tkinter as tk
import tkinter.ttk as ttk

from ..resources.language import Text


class GraphNameAndStyle:
    def __init__(self, parent, index) -> None:

        self.parent = parent
        self.index = index

        self.top = tk.Toplevel()
        if self.index >= len(self.parent.DoseFigureHandler.plots):
            self.top.destroy()
        self.geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]
        self.width = 225
        self.height = 110
        if self.index == "X":
            self.width = 130

        self.top.geometry(
            f"{self.width}x{self.height}+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )
        self.top.overrideredirect(1)

        self.top.bind("<Return>", self.submit)
        self.top.bind("<Escape>", self.top.destroy)
        self.enterlabel = ttk.Label(
            self.top, text=Text().changefilename[self.parent.lang]
        )
        self.enterbox = ttk.Entry(self.top, width=15)

        self.enterlabel.grid(row=0, column=0, padx=(5, 5), pady=(2, 2))
        self.enterbox.grid(row=1, column=0, padx=(5, 5), pady=(2, 2))
        self.submitbutton = ttk.Button(
            self.top, text=Text().submit[self.parent.lang], command=self.submit,
        )
        self.submitbutton.grid(row=2, column=0, padx=(5, 5), pady=(2, 2))

        if index != "X":
            self.enterbox.insert(
                tk.END, string=self.parent.DoseFigureHandler.plots[index].filename
            )
            self.radiovar = tk.StringVar()
            self.radiovar.set(self.parent.DoseFigureHandler.marker[index])
            self.values = [".", "-", "o--"]
            self.styles = [
                Text().dot[self.parent.lang],
                Text().dash[self.parent.lang],
                Text().dashdot[self.parent.lang],
            ]
            self.radiobuttons = [
                ttk.Radiobutton(
                    self.top,
                    variable=self.radiovar,
                    value=self.values[i],
                    text=self.styles[i],
                )
                for i in range(3)
            ]

            for i, button in enumerate(self.radiobuttons):
                button.grid(row=i, column=1, padx=(0, 5), sticky=tk.W, pady=(2, 2))
        else:
            xlabel = self.parent.DoseFigureHandler.xaxisname
            if xlabel == None:
                xlabel = self.parent.DoseFigureHandler.xlabel
            self.enterbox.insert(tk.END, string=xlabel)

        self.enterbox.focus()
        self.lift()

    def lift(self):
        self.new_geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]

        self.top.lift()
        self.top.after(100, self.lift)
        if self.new_geometry == self.geometry:
            return

        self.geometry = self.new_geometry
        self.top.geometry(
            f"{self.width}x{self.height}+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )

    def submit(self, event=None):

        self.top.unbind("<Return>")

        if self.index == "X":
            self.parent.DoseFigureHandler.xaxisname = self.enterbox.get()

        else:
            self.parent.DoseFigureHandler.plots[
                self.index
            ].filename = self.enterbox.get()
            self.parent.DoseFigureHandler.marker[self.index] = self.radiovar.get()
            self.parent.profile.set_attribute(
                "markers", self.parent.DoseFigureHandler.marker
            )

        self.top.destroy()



import tkinter as tk
import tkinter.ttk as ttk

import numpy as np

from ..functions import dp, pdd
from ..resources.language import Text


class Measurement:
    def __init__(self, filepath, parent):

        self.filepath = filepath
        self.parent = parent
        self.filename = self.filepath.split("/")[-1][:-4]

        data = np.loadtxt(self.filepath, unpack=True)

        self.axis, self.dose = data[0], data[1]
        self.normpoint = max(self.dose)

        try:
            self.std_dev = data[2]
        except IndexError:
            self.std_dev = []

        self.axis = self.axis.tolist()

        self.axis = {True: self.axis[len(self.axis) // 2 :], False: self.axis}
        self.dose = {True: self.dose[len(self.dose) // 2 :], False: self.dose}
        if self.std_dev != None:
            self.std_dev = self.std_dev = {
                True: self.std_dev[len(self.std_dev) // 2 :],
                False: self.std_dev,
            }

        else:
            self.std_dev = {
                True: self.std_dev,
                False: self.std_dev,
            }

        try:
            self.direction = "Z"
            pdd.calculate_parameters(
                np.array(self.axis[False]),
                self.dose[False] / max(self.dose[False]),
                [],
            )
        except Exception:
            try:
                self.direction = "X"
                dp.calculate_parameters(
                    np.array(self.axis[False]),
                    self.dose[False] / max(self.dose[False]),
                    [],
                )
                dialog = GetType(self, self.parent, "NoPDD")
                self.parent.parent.wait_window(dialog.top)
            except Exception:
                dialog = GetType(self, self.parent, "")
                self.parent.parent.wait_window(dialog.top)

    def params(self):
        if self.direction == "Z":
            return pdd.calculate_parameters(
                np.array(self.axis[False]),
                self.dose[False] / max(self.dose[False]),
                [],
            )
        else:
            params = dp.calculate_parameters(
                self.axis[False], self.dose[False] / max(self.dose[False])
            )
            self.cax = params[1]
            return params


class GetType:
    def __init__(self, measurement, parent, opt: str):

        self.measurement = measurement
        self.parent = parent
        self.top = tk.Toplevel()
        self.geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]
        self.width = 300
        if opt == "NoPDD":
            self.width = 200
        self.top.geometry(
            f"{self.width}x60+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )
        self.top.overrideredirect(1)

        self.xbutton = ttk.Button(self.top, text="X", command=self.x)
        self.ybutton = ttk.Button(self.top, text="Y", command=self.y)
        self.zbutton = ttk.Button(self.top, text="Z", command=self.z)
        self.dplabel = ttk.Label(self.top, text=Text().dp[self.parent.lang])
        self.pddlabel = ttk.Label(self.top, text=Text().pdd[self.parent.lang])

        self.dplabel.grid(row=0, column=0, columnspan=2, padx=(5, 5))
        self.xbutton.grid(column=0, row=1, padx=(5, 0))
        self.ybutton.grid(column=1, row=1, padx=(5, 5))
        if opt != "NoPDD":
            self.pddlabel.grid(row=0, column=2, padx=(0, 5))
            self.zbutton.grid(column=2, row=1, padx=(0, 5))
        self.lift()

    def lift(self):
        self.new_geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]

        self.top.lift()
        self.top.after(100, self.lift)
        if self.new_geometry == self.geometry:
            return

        self.geometry = self.new_geometry
        self.top.geometry(
            f"{self.width}x60+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )

    def x(self):
        self.measurement.direction = "X"
        self.top.destroy()

    def y(self):
        self.measurement.direction = "Y"
        self.top.destroy()

    def z(self):
        self.measurement.direction = "Z"
        self.top.destroy()



# -*- coding: utf-8 -*-

'''Python wrapper for the tkdnd tk extension.
The tkdnd extension provides an interface to native, platform specific
drag and drop mechanisms. Under Unix the drag & drop protocol in use is
the XDND protocol version 5 (also used by the Qt toolkit, and the KDE and
GNOME desktops). Under Windows, the OLE2 drag & drop interfaces are used.
Under Macintosh, the Cocoa drag and drop interfaces are used.

Once the TkinterDnD2 package is installed, it is safe to do:

from TkinterDnD2 import *

This will add the classes TkinterDnD.Tk and TkinterDnD.TixTk to the global
namespace, plus the following constants:
PRIVATE, NONE, ASK, COPY, MOVE, LINK, REFUSE_DROP,
DND_TEXT, DND_FILES, DND_ALL, CF_UNICODETEXT, CF_TEXT, CF_HDROP,
FileGroupDescriptor, FileGroupDescriptorW

Drag and drop for the application can then be enabled by using one of the
classes TkinterDnD.Tk() or (in case the tix extension shall be used)
TkinterDnD.TixTk() as application main window instead of a regular
tkinter.Tk() window. This will add the drag-and-drop specific methods to the
Tk window and all its descendants.'''

try:
    import Tkinter as tkinter
    import Tix as tix
except ImportError:
    import tkinter
    from tkinter import tix

TkdndVersion = None

def _require(tkroot):
    '''Internal function.'''
    global TkdndVersion
    try:
        TkdndVersion = tkroot.tk.call('package', 'require', 'tkdnd')
    except tkinter.TclError:
        raise RuntimeError('Unable to load tkdnd library.')
    return TkdndVersion

class DnDEvent:
    """Internal class.
    Container for the properties of a drag-and-drop event, similar to a
    normal tkinter.Event.
    An instance of the DnDEvent class has the following attributes:
        action (string)
        actions (tuple)
        button (int)
        code (string)
        codes (tuple)
        commonsourcetypes (tuple)
        commontargettypes (tuple)
        data (string)
        name (string)
        types (tuple)
        modifiers (tuple)
        supportedsourcetypes (tuple)
        sourcetypes (tuple)
        type (string)
        supportedtargettypes (tuple)
        widget (widget instance)
        x_root (int)
        y_root (int)
    Depending on the type of DnD event however, not all attributes may be set.
    """
    pass

class DnDWrapper:
    '''Internal class.'''
    # some of the percent substitutions need to be enclosed in braces
    # so we can use splitlist() to convert them into tuples
    _subst_format_dnd = ('%A', '%a', '%b', '%C', '%c', '{%CST}',
                         '{%CTT}', '%D', '%e', '{%L}', '{%m}', '{%ST}',
                         '%T', '{%t}', '{%TT}', '%W', '%X', '%Y')
    _subst_format_str_dnd = " ".join(_subst_format_dnd)
    tkinter.BaseWidget._subst_format_dnd = _subst_format_dnd
    tkinter.BaseWidget._subst_format_str_dnd = _subst_format_str_dnd

    def _substitute_dnd(self, *args):
        """Internal function."""
        if len(args) != len(self._subst_format_dnd):
            return args
        def getint_event(s):
            try:
                return int(s)
            except ValueError:
                return s
        def splitlist_event(s):
            try:
                return self.tk.splitlist(s)
            except ValueError:
                return s
        # valid percent substitutions for DnD event types
        # (tested with tkdnd-2.8 on debian jessie):
        # <<DragInitCmd>> : %W, %X, %Y %e, %t
        # <<DragEndCmd>> : %A, %W, %e
        # <<DropEnter>> : all except : %D (always empty)
        # <<DropLeave>> : all except %D (always empty)
        # <<DropPosition>> :all except %D (always empty)
        # <<Drop>> : all
        A, a, b, C, c, CST, CTT, D, e, L, m, ST, T, t, TT, W, X, Y = args
        ev = DnDEvent()
        ev.action = A
        ev.actions = splitlist_event(a)
        ev.button = getint_event(b)
        ev.code = C
        ev.codes = splitlist_event(c)
        ev.commonsourcetypes = splitlist_event(CST)
        ev.commontargettypes = splitlist_event(CTT)
        ev.data = D
        ev.name = e
        ev.types = splitlist_event(L)
        ev.modifiers = splitlist_event(m)
        ev.supportedsourcetypes = splitlist_event(ST)
        ev.sourcetypes = splitlist_event(t)
        ev.type = T
        ev.supportedtargettypes = splitlist_event(TT)
        try:
            ev.widget = self.nametowidget(W)
        except KeyError:
            ev.widget = W
        ev.x_root = getint_event(X)
        ev.y_root = getint_event(Y)
        return (ev,)
    tkinter.BaseWidget._substitute_dnd = _substitute_dnd

    def _dnd_bind(self, what, sequence, func, add, needcleanup=True):
        """Internal function."""
        if isinstance(func, str):
            self.tk.call(what + (sequence, func))
        elif func:
            funcid = self._register(func, self._substitute_dnd, needcleanup)
            # FIXME: why doesn't the "return 'break'" mechanism work here??
            #cmd = ('%sif {"[%s %s]" == "break"} break\n' % (add and '+' or '',
            #                              funcid, self._subst_format_str_dnd))
            cmd = '%s%s %s' %(add and '+' or '', funcid,
                                    self._subst_format_str_dnd)
            self.tk.call(what + (sequence, cmd))
            return funcid
        elif sequence:
            return self.tk.call(what + (sequence,))
        else:
            return self.tk.splitlist(self.tk.call(what))
    tkinter.BaseWidget._dnd_bind = _dnd_bind

    def dnd_bind(self, sequence=None, func=None, add=None):
        '''Bind to this widget at drag and drop event SEQUENCE a call
        to function FUNC.
        SEQUENCE may be one of the following:
        <<DropEnter>>, <<DropPosition>>, <<DropLeave>>, <<Drop>>,
        <<Drop:type>>, <<DragInitCmd>>, <<DragEndCmd>> .
        The callbacks for the <Drop*>> events, with the exception of
        <<DropLeave>>, should always return an action (i.e. one of COPY,
        MOVE, LINK, ASK or PRIVATE).
        The callback for the <<DragInitCmd>> event must return a tuple
        containing three elements: the drop action(s) supported by the
        drag source, the format type(s) that the data can be dropped as and
        finally the data that shall be dropped. Each of these three elements
        may be a tuple of strings or a single string.'''
        return self._dnd_bind(('bind', self._w), sequence, func, add)
    tkinter.BaseWidget.dnd_bind = dnd_bind

    def drag_source_register(self, button=None, *dndtypes):
        '''This command will register SELF as a drag source.
        A drag source is a widget than can start a drag action. This command
        can be executed multiple times on a widget.
        When SELF is registered as a drag source, optional DNDTYPES can be
        provided. These DNDTYPES will be provided during a drag action, and
        it can contain platform independent or platform specific types.
        Platform independent are DND_Text for dropping text portions and
        DND_Files for dropping a list of files (which can contain one or
        multiple files) on SELF. However, these types are
        indicative/informative. SELF can initiate a drag action with even a
        different type list. Finally, button is the mouse button that will be
        used for starting the drag action. It can have any of the values 1
        (left mouse button), 2 (middle mouse button - wheel) and 3
        (right mouse button). If button is not specified, it defaults to 1.'''
        # hack to fix a design bug from the first version
        if button is None:
            button = 1
        else:
            try:
                button = int(button)
            except ValueError:
                # no button defined, button is actually
                # something like DND_TEXT
                dndtypes = (button,) + dndtypes
                button = 1
        self.tk.call(
                'tkdnd::drag_source', 'register', self._w, dndtypes, button)
    tkinter.BaseWidget.drag_source_register = drag_source_register

    def drag_source_unregister(self):
        '''This command will stop SELF from being a drag source. Thus, window
        will stop receiving events related to drag operations. It is an error
        to use this command for a window that has not been registered as a
        drag source with drag_source_register().'''
        self.tk.call('tkdnd::drag_source', 'unregister', self._w)
    tkinter.BaseWidget.drag_source_unregister = drag_source_unregister

    def drop_target_register(self, *dndtypes):
        '''This command will register SELF as a drop target. A drop target is
        a widget than can accept a drop action. This command can be executed
        multiple times on a widget. When SELF is registered as a drop target,
        optional DNDTYPES can be provided. These types list can contain one or
        more types that SELF will accept during a drop action, and it can
        contain platform independent or platform specific types. Platform
        independent are DND_Text for dropping text portions and DND_Files for
        dropping a list of files (which can contain one or multiple files) on
        SELF.'''
        self.tk.call('tkdnd::drop_target', 'register', self._w, dndtypes)
    tkinter.BaseWidget.drop_target_register = drop_target_register

    def drop_target_unregister(self):
        '''This command will stop SELF from being a drop target. Thus, SELF
        will stop receiving events related to drop operations. It is an error
        to use this command for a window that has not been registered as a
        drop target with drop_target_register().'''
        self.tk.call('tkdnd::drop_target', 'unregister', self._w)
    tkinter.BaseWidget.drop_target_unregister = drop_target_unregister

    def platform_independent_types(self, *dndtypes):
        '''This command will accept a list of types that can contain platform
        independnent or platform specific types. A new list will be returned,
        where each platform specific type in DNDTYPES will be substituted by
        one or more platform independent types. Thus, the returned list may
        have more elements than DNDTYPES.'''
        return self.tk.split(self.tk.call(
                            'tkdnd::platform_independent_types', dndtypes))
    tkinter.BaseWidget.platform_independent_types = platform_independent_types

    def platform_specific_types(self, *dndtypes):
        '''This command will accept a list of types that can contain platform
        independnent or platform specific types. A new list will be returned,
        where each platform independent type in DNDTYPES will be substituted
        by one or more platform specific types. Thus, the returned list may
        have more elements than DNDTYPES.'''
        return self.tk.split(self.tk.call(
                            'tkdnd::platform_specific_types', dndtypes))
    tkinter.BaseWidget.platform_specific_types = platform_specific_types

    def get_dropfile_tempdir(self):
        '''This command will return the temporary directory used by TkDND for
        storing temporary files. When the package is loaded, this temporary
        directory will be initialised to a proper directory according to the
        operating system. This default initial value can be changed to be the
        value of the following environmental variables:
        TKDND_TEMP_DIR, TEMP, TMP.'''
        return self.tk.call('tkdnd::GetDropFileTempDirectory')
    tkinter.BaseWidget.get_dropfile_tempdir = get_dropfile_tempdir

    def set_dropfile_tempdir(self, tempdir):
        '''This command will change the temporary directory used by TkDND for
        storing temporary files to TEMPDIR.'''
        self.tk.call('tkdnd::SetDropFileTempDirectory', tempdir)
    tkinter.BaseWidget.set_dropfile_tempdir = set_dropfile_tempdir

#######################################################################
####      The main window classes that enable Drag & Drop for      ####
####      themselves and all their descendant widgets:             ####
#######################################################################

class Tk(tkinter.Tk, DnDWrapper):
    '''Creates a new instance of a tkinter.Tk() window; all methods of the
    DnDWrapper class apply to this window and all its descendants.'''
    def __init__(self, *args, **kw):
        tkinter.Tk.__init__(self, *args, **kw)
        self.TkdndVersion = _require(self)

class TixTk(tix.Tk, DnDWrapper):
    '''Creates a new instance of a tix.Tk() window; all methods of the
    DnDWrapper class apply to this window and all its descendants.'''
    def __init__(self, *args, **kw):
        tix.Tk.__init__(self, *args, **kw)
        self.TkdndVersion = _require(self)



import os
import time
import tkinter as tk
from pathlib import Path
from tkinter import filedialog as fd
from tkinter import simpledialog as sd
from tkinter.colorchooser import askcolor

import pynput
from PIL import Image, ImageTk

from ..resources.info import show_info
from ..resources.language import Text
from .dose_figure_handler import DoseFigureHandler
from .gamma import ChangeGamma
from .nameandstyle import GraphNameAndStyle
from .profile import ProfileHandler
from .recent_files import RecentFileManager
from .table import SimpleTable
from .xrangeslider import XRangeSlider


class MainApplication(tk.Frame):
    def __init__(self, parent):

        tk.Frame.__init__(self, parent)
        self.starttime = time.time()

        self.parent = parent

        self.logocanvas = tk.Canvas(self)
        self.logocanvas.pack(side=tk.TOP, fill="both", expand=True)
        self.center = [
            (self.winfo_screenwidth() // 2 + 50) // 2,
            (self.parent.winfo_screenheight() // 2) // 2,
        ]
        self.logo = ImageTk.PhotoImage(
            Image.open(
                os.path.join(
                    os.path.dirname(os.path.realpath(__file__)),
                    "..",
                    "resources",
                    "icon.png",
                )
            ).resize(
                (self.parent.minsize()[0] // 4, (self.parent.minsize()[0] // 4)),
                Image.LANCZOS,
            ),
            master=self.logocanvas,
        )
        self.logo_on_canvas = self.logocanvas.create_image(
            self.center[0], self.center[1], anchor=tk.CENTER, image=self.logo,
        )
        self.logocanvas.image = self.logo
        self.logocanvas.itemconfig(self.logo_on_canvas, image=self.logocanvas.image)

        self.parent.protocol("WM_DELETE_WINDOW", lambda: self.save_graph(True))

        # Read settings from profile.json and initialize the necessary variables
        self.direction = None
        self.saved = True
        self.menuflag = False
        self.xlimmenu = False
        self.canvas = tk.Canvas(self)
        self.photoimage = None
        self.image_on_canvas = None
        self.canvas.image = None
        self.current_file = None
        self.filenames = []
        self.rename_boxes = []
        self.new_limits = []
        self.current_event_width = -1
        self.current_event_height = -1

        self.profile = ProfileHandler()
        self.lang = self.profile.get_attribute("language")

        self.langvar = tk.StringVar()
        self.langvar.set(self.lang)
        self.text = Text()

        self.autostartdark = tk.StringVar()
        self.autostartdark.set(self.profile.get_attribute("color_scheme"))

        self.dark = tk.BooleanVar()
        self.dark.set(False)

        self.norm = tk.BooleanVar()
        self.norm.set(bool(self.profile.get_attribute("normalize")))

        self.errorbars = tk.BooleanVar()
        self.errorbars.set(True)

        self.caxcorrection = tk.BooleanVar()
        self.caxcorrection.set(False)

        self.diffplot = tk.BooleanVar()
        self.diffplot.set(False)

        self.zoom = tk.BooleanVar()
        self.zoom.set(bool(self.profile.get_attribute("zoom")))

        self.half = tk.BooleanVar()
        self.half.set(bool(self.profile.get_attribute("halfview")))

        self.calcparams = tk.BooleanVar()
        self.calcparams.set(False)

        self.tablevar = tk.BooleanVar()
        self.tablevar.set(False)

        self.fullscreen = tk.BooleanVar()
        self.fullscreen.set(bool(self.profile.get_attribute("fullscreen")))

        self.axlims = tk.IntVar()
        self.axlims.set(0)

        self.normvaluemenu = tk.StringVar()
        self.normvaluemenu.set("max")

        self.errlimval = tk.StringVar()
        self.errlimval.set("absolute")

        self.slidervars = [tk.DoubleVar(), tk.DoubleVar()]
        self.current_limits = []
        self.initial_limits = []

        self.DoseFigureHandler = DoseFigureHandler(self)

        self.RecentFileManager = RecentFileManager(self)

        # Keybinding definitions
        self.parent.bind("<Configure>", self.handle_configure)
        self.parent.bind("<Control-e>", lambda type: self.load_file("egs"))
        self.parent.bind("<Control-m>", lambda type: self.load_file("measurement"))
        self.parent.bind("<Control-o>", lambda type: self.load_file("simulation"))
        self.parent.bind("<Control-s>", self.save_graph)
        self.parent.bind("<Control-p>", lambda type: self.load_file("ptw"))
        self.parent.bind("<Escape>", self.close_file)
        self.parent.bind("<Control-z>", self.remove_last_addition)
        self.parent.bind("<F11>", self.toggle_fullscreen)
        self.parent.bind("<MouseWheel>", self.change_x_limits)
        self.parent.bind(
            "<Control-Alt-Up>", lambda boolean1: self.change_errlims(True, True)
        )
        self.parent.bind(
            "<Control-Alt-Down>", lambda boolean1: self.change_errlims(True, False),
        )
        self.parent.bind(
            "<Control-Shift-Up>", lambda boolean1: self.change_errlims(False, True),
        )
        self.parent.bind(
            "<Control-Shift-Down>", lambda boolean1: self.change_errlims(False, False),
        )
        self.parent.bind("<Control-Up>", lambda boolean: self.change_marker_size(True))
        self.parent.bind(
            "<Control-Down>", lambda boolean: self.change_marker_size(False)
        )

        self.parent.bind(
            "<Control-Right>", lambda boolean: self.change_line_width(True)
        )
        self.parent.bind(
            "<Control-Left>", lambda boolean: self.change_line_width(False)
        )

        # Menubar definitions

        self.menubar = tk.Menu(self.parent, tearoff=False)
        self.filemenu = tk.Menu(self.menubar, tearoff=False)
        self.addmeasuremenu = tk.Menu(self.menubar, tearoff=False)
        self.addmeasuremenu.add_command(
            label=self.text.ptw[self.lang],
            command=lambda: self.load_file("ptw"),
            accelerator="Ctrl+P",
        )
        self.addmeasuremenu.add_command(
            label=self.text.measurement[self.lang],
            command=lambda: self.load_file("measurement"),
            accelerator="Ctrl+M",
        )

        self.addsimmenu = tk.Menu(self.menubar, tearoff=False)

        self.addsimmenu.add_command(
            label="TOPAS",
            command=lambda: self.load_file("simulation"),
            accelerator="Ctrl+O",
        )
        self.addsimmenu.add_command(
            label="3Ddose", command=lambda: self.load_file("egs"), accelerator="Ctrl+E",
        )

        self.filemenu.add_cascade(
            label=self.text.loadsim[self.lang], menu=self.addsimmenu
        )
        self.filemenu.add_cascade(
            label=self.text.loadmeasurement[self.lang], menu=self.addmeasuremenu
        )

        self.recentmenu = tk.Menu(self.menubar, tearoff=False)
        self.filemenu.add_cascade(
            label=self.text.recent[self.lang], menu=self.recentmenu
        )
        self.RecentFileManager.add_files_to_menu()

        self.filemenu.add_separator()
        self.filemenu.add_command(
            label=self.text.save[self.lang],
            command=self.save_graph,
            accelerator="Ctrl+S",
        )
        self.filemenu.add_command(
            label=self.text.close[self.lang],
            command=self.close_file,
            accelerator="Esc",
        )
        self.filemenu.add_separator()
        self.filemenu.add_command(
            label=self.text.end[self.lang], command=lambda: self.save_graph(True)
        )
        self.filemenu.entryconfig(4, state=tk.DISABLED)
        self.filemenu.entryconfig(5, state=tk.DISABLED)
        self.addmenu = tk.Menu(self.menubar, tearoff=False)
        self.addmenu.add_cascade(
            label=self.text.simulation[self.lang], menu=self.addsimmenu
        )
        self.addmenu.add_cascade(
            label=self.text.measurement[self.lang], menu=self.addmeasuremenu
        )
        self.addmenu.add_separator()
        self.addmenu.add_command(
            label=self.text.revert[self.lang],
            command=self.remove_last_addition,
            accelerator="Ctrl+Z",
        )
        self.menubar.add_cascade(label=self.text.file[self.lang], menu=self.filemenu)
        self.viewmenu = tk.Menu(self.menubar, tearoff=False)
        self.viewmenu.add_checkbutton(
            label=self.text.light[self.lang],
            variable=self.dark,
            command=lambda: self.switchtheme("light"),
            onvalue=0,
            offvalue=1,
        )
        self.viewmenu.add_checkbutton(
            label=self.text.dark[self.lang],
            variable=self.dark,
            command=lambda: self.switchtheme("dark"),
            onvalue=1,
            offvalue=0,
        )

        self.viewmenu.add_checkbutton(
            label=self.text.fullscreen[self.lang],
            variable=self.fullscreen,
            command=self.toggle_fullscreen,
            onvalue=1,
            offvalue=0,
            accelerator="F11",
        )

        self.menubar.add_cascade(label=self.text.view[self.lang], menu=self.viewmenu)
        self.optionsmenu = tk.Menu(self.menubar, tearoff=False)
        self.optionsmenu.add_checkbutton(
            label=self.text.startdark[self.lang],
            variable=self.autostartdark,
            command=lambda: self.profile.set_attribute(
                "color_scheme", f"{self.autostartdark.get()}"
            ),
            onvalue="dark",
            offvalue="light",
        )
        self.langmenu = tk.Menu(self.menubar, tearoff=False)
        self.langmenu.add_radiobutton(
            label=self.text.german[self.lang],
            command=lambda: self.set_language(self.langvar.get()),
            variable=self.langvar,
            value="de",
        )
        self.langmenu.add_radiobutton(
            label=self.text.english[self.lang],
            command=lambda: self.set_language(self.langvar.get()),
            variable=self.langvar,
            value="en",
        )
        self.optionsmenu.add_cascade(
            label=self.text.language[self.lang], menu=self.langmenu
        )
        self.menubar.add_cascade(
            label=self.text.options[self.lang], menu=self.optionsmenu
        )

        self.normmenu = tk.Menu(self.menubar, tearoff=False)

        self.markersizemenu = tk.Menu(self.menubar, tearoff=False)
        self.markersizemenu.add_command(
            label=Text().increase[self.lang],
            command=lambda: self.change_marker_size(True),
            accelerator="Ctrl + ↑",
        )

        self.markersizemenu.add_command(
            label=Text().decrease[self.lang],
            command=lambda: self.change_marker_size(False),
            accelerator="Ctrl + ↓",
        )

        self.markerlinemenu = tk.Menu(self.menubar, tearoff=False)
        self.markerlinemenu.add_command(
            label=Text().increase[self.lang],
            command=lambda: self.change_line_width(True),
            accelerator="Ctrl + →",
        )

        self.markerlinemenu.add_command(
            label=Text().decrease[self.lang],
            command=lambda: self.change_line_width(False),
            accelerator="Ctrl + ←",
        )

        self.normmenu.add_cascade(
            label=Text().markerline[self.lang], menu=self.markerlinemenu
        )
        self.normmenu.add_cascade(
            label=Text().marker[self.lang], menu=self.markersizemenu
        )

        self.normmenu.add_command(
            label=self.text.resetcolors[self.lang], command=self.reset_colors
        )

        self.normmenu.add_command(
            label=self.text.resetmarkers[self.lang], command=self.reset_markers
        )

        self.normmenu.add_separator()

        self.normalizemenu = tk.Menu(self.menubar, tearoff=False)
        self.normalizemenu.add_checkbutton(
            label=self.text.normalize[self.lang],
            command=self.normalize,
            variable=self.norm,
        )
        self.normalize_cascade = tk.Menu(self.menubar, tearoff=False)
        self.normalize_cascade.add_radiobutton(
            label=self.text.maximum[self.lang],
            variable=self.normvaluemenu,
            value="max",
            command=self.change_normalization,
        )
        self.normalize_cascade.add_radiobutton(
            label=self.text.plateau[self.lang],
            variable=self.normvaluemenu,
            value="plateau",
            command=self.change_normalization,
        )
        self.normalize_cascade.add_radiobutton(
            label=self.text.centeraxis[self.lang],
            variable=self.normvaluemenu,
            value="center",
            command=self.change_normalization,
        )
        self.normalizemenu.add_cascade(
            label=self.text.choosenormalization[self.lang], menu=self.normalize_cascade
        )

        self.normmenu.add_cascade(
            label=self.text.normalization[self.lang], menu=self.normalizemenu
        )
        self.normmenu.add_checkbutton(
            label=self.text.errorbars[self.lang],
            command=self.show_errorbars,
            variable=self.errorbars,
        )
        self.normmenu.add_checkbutton(
            label=self.text.zoom[self.lang], command=self.zoomgraph, variable=self.zoom,
        )

        self.normmenu.add_checkbutton(
            label=self.text.half[self.lang], command=self.halfgraph, variable=self.half,
        )
        self.normmenu.add_separator()

        self.parammenu = tk.Menu(self.menubar, tearoff=False)
        self.parammenu.add_checkbutton(
            label=self.text.adddescriptors[self.lang],
            command=self.calculate_parameters,
            variable=self.calcparams,
        )
        self.parammenu.add_checkbutton(
            label=self.text.showtable[self.lang],
            command=self.show_table,
            variable=self.tablevar,
        )

        self.normmenu.add_cascade(
            label=self.text.calcparams[self.lang], menu=self.parammenu
        )

        self.normmenu.add_checkbutton(
            label=self.text.caxcorrection[self.lang],
            command=self.correct_caxdev,
            variable=self.caxcorrection,
        )
        self.normmenu.add_separator()
        self.normmenu.add_checkbutton(
            label=self.text.differenceplot[self.lang],
            command=self.differenceplot,
            variable=self.diffplot,
        )

        self.gammamenu = tk.Menu(self.menubar, tearoff=False)
        self.gammaacc = tk.StringVar()
        self.gammadist = 2
        self.gammathreshold = 2
        self.gammaacc.set(f"{self.gammathreshold}%/{self.gammadist}mm")
        self.gammamenu.add_command(
            label=self.text.calculate[self.lang], command=self.gamma
        )
        self.gammamenu.add_command(
            label=self.text.gamma[self.lang],
            accelerator=self.gammaacc.get(),
            command=self.changegamma,
        )

        self.normmenu.add_cascade(
            label=self.text.gammamenu[self.lang], menu=self.gammamenu
        )

        self.errlimmenu = tk.Menu(self.menubar, tearoff=False)
        self.errlimmenu.add_command(
            label=Text().increaseupper[self.lang],
            command=lambda: self.change_errlims(True, True),
            accelerator="Ctrl + +",
        )

        self.errlimmenu.add_command(
            label=Text().decreaseupper[self.lang],
            command=lambda: self.change_errlims(True, False),
            accelerator="Ctrl + -",
        )
        self.errlimmenu.add_command(
            label=Text().increselower[self.lang],
            command=lambda: self.change_errlims(False, True),
            accelerator="Ctrl + Alt + +",
        )
        self.errlimmenu.add_command(
            label=Text().decreaselower[self.lang],
            command=lambda: self.change_errlims(False, False),
            accelerator="Ctrl + Alt + -",
        )

        self.errlimvalmenu = tk.Menu(self.menubar, tearoff=False)
        self.errlimvalmenu.add_radiobutton(
            label=self.text.percentage[self.lang],
            variable=self.errlimval,
            value="percentage",
            command=self.changeerrdisplay,
        )
        self.errlimvalmenu.add_radiobutton(
            label=self.text.absolute[self.lang],
            variable=self.errlimval,
            value="absolute",
            command=self.changeerrdisplay,
        )

        self.errlimmenu.add_cascade(
            label=self.text.changeerr[self.lang], menu=self.errlimvalmenu
        )

        self.normmenu.add_cascade(
            label=Text().errlimmenu[self.lang], menu=self.errlimmenu
        )

        self.normmenu.entryconfig(13, state=tk.DISABLED)
        self.normmenu.entryconfig(14, state=tk.DISABLED)
        self.normmenu.entryconfig(15, state=tk.DISABLED)

        self.menubar.add_command(label=self.text.about[self.lang], command=self.about)

        self.parent.config(menu=self.menubar)

        self.parent.title(self.text.window_title[self.lang])

        self.pack(side="top", fill="both", expand=True)
        self.frame2 = tk.Frame(self.parent, bg="red")

        self.parent.attributes("-fullscreen", self.fullscreen.get())
        self.autostart()

    def about(self):
        show_info(self.parent, self.lang, self.dark.get())

    def autostart(self):

        """Sets the theme according to the profile.json
        """

        if self.autostartdark.get() == "light":
            self.dark.set(False)
            self.switchtheme("light")
        else:
            self.dark.set(True)
            self.switchtheme("dark")

        return

    def switchtheme(self, theme):

        """Switches between light and dark theme
        """

        if theme == "light":
            self.parent.tk.call("set_theme", "light")
            self.logocanvas.configure(background="#ffffff")
            self.parent.configure(background="#ffffff")

        else:
            # Set dark theme
            self.parent.tk.call("set_theme", "dark")
            self.parent.configure(background="#363636")
            self.logocanvas.configure(background="#333333")

        if self.filenames != []:
            self.logocanvas.pack_forget()
            self.show_preview()

        return

    def toggle_fullscreen(self, event=None):

        """Toggle between fullscreen and normal view
        """

        self.parent.attributes("-fullscreen", not self.parent.attributes("-fullscreen"))
        self.fullscreen.set(self.parent.attributes("-fullscreen"))
        self.profile.set_attribute("fullscreen", bool(self.fullscreen.get()))

    def change_x_limits(self, event=None):

        if event.delta > 0:

            if self.DoseFigureHandler.plots[0].direction == "Z":
                self.slidervars[1].set(self.slidervars[1].get() - 5)
            else:

                if self.slidervars[0].get() >= 0:
                    self.slidervars[0].set(self.slidervars[0].get() - 5)
                else:
                    self.slidervars[0].set(self.slidervars[0].get() + 5)

                if self.slidervars[1].get() >= 0:
                    self.slidervars[1].set(self.slidervars[1].get() - 5)
                else:
                    self.slidervars[1].set(self.slidervars[1].get() + 5)
        elif event.delta < 0:

            if self.DoseFigureHandler.plots[0].direction == "Z":
                self.slidervars[1].set(self.slidervars[1].get() + 5)
            else:

                if self.slidervars[0].get() >= 0:
                    self.slidervars[0].set(self.slidervars[0].get() + 5)
                else:
                    self.slidervars[0].set(self.slidervars[0].get() - 5)

                if self.slidervars[1].get() >= 0:
                    self.slidervars[1].set(self.slidervars[1].get() + 5)
                else:
                    self.slidervars[1].set(self.slidervars[1].get() - 5)

        self.current_limits = [self.slidervars[0].get(), self.slidervars[1].get()]

        self.show_preview()

        return

    def change_xrange(self):

        self.parent.unbind("<MouseWheel>")
        for i, var in enumerate(self.initial_limits):
            if var < 0 and var < self.current_limits[i]:
                self.current_limits[i] = var

            if var >= 0 and var > self.current_limits[i]:
                self.current_limits[i] = var

        self.xlimmenu = True
        self.slider = XRangeSlider(self, self.slidervars, self.current_limits)

        return

    def set_language(self, language):

        """Sets the desired language and reinitiates the program
        """

        plots = self.DoseFigureHandler.plots
        filenames = self.filenames
        dark = self.dark.get()
        axlims = self.axlims.get()
        calcparams = self.calcparams.get()
        caxcorrection = self.caxcorrection.get()
        normalize = self.norm.get()
        errorbars = self.errorbars.get()
        diffplot = self.diffplot.get()
        errlimmin = self.DoseFigureHandler.errlimmin
        errlimmax = self.DoseFigureHandler.errlimmax
        initial_limits = self.initial_limits
        current_limits = [self.slidervars[0].get(), self.slidervars[1].get()]
        table = self.tablevar.get()
        dark = self.dark.get()
        try:
            self.slider.submit()
            self.slider.window.destroy()
            self.slider = None
        except Exception:
            pass
        self.pack_forget()
        self.parent.config(menu=None)
        self.profile.set_attribute("language", language)
        self.__init__(self.parent)
        if plots != []:
            self.filenames = filenames
            self.DoseFigureHandler.plots = plots
            self.dark.set(dark)
            self.tablevar.set(table)
            self.axlims.set(axlims)
            self.calcparams.set(calcparams)
            self.caxcorrection.set(caxcorrection)
            self.norm.set(normalize)
            self.errorbars.set(errorbars)
            self.diffplot.set(diffplot)
            self.current_limits = current_limits
            self.initial_limits = initial_limits
            self.slidervars[0].set(self.current_limits[0])
            self.slidervars[1].set(self.current_limits[1])
            self.DoseFigureHandler.diffplot = diffplot
            self.DoseFigureHandler.errlimmin = errlimmin
            self.DoseFigureHandler.errlimmax = errlimmax
            if dark == True:
                self.switchtheme("dark")
                self.dark.set(True)
            else:
                self.switchtheme("light")
                self.dark.set(False)
            self.show_preview()
            if table == True:
                if self.direction != "Z":
                    self.table.set(0, 1, self.text.fwhm[self.lang], self.fontsize)

        return

    def load_file(self, type, event=None):

        """Loads measurement or simulation data to be displayed
        """

        if type == "simulation":
            filetypes = [(self.text.topas[self.lang], [".csv", ".bin"])]
        if type == "egs":
            filetypes = [(self.text.egs[self.lang], [".3ddose"])]
        elif type == "measurement":
            filetypes = [(self.text.measurementdata[self.lang], ["txt", ".csv"])]
        elif type == "ptw":
            filetypes = [(self.text.ptw[self.lang], ".mcc")]

        try:
            initialdir = os.path.dirname(
                self.profile.get_attribute("recent_files")[0][0]
            )
            print(initialdir)
        except IndexError:
            initialdir = Path.home()

        self.current_file = fd.askopenfilenames(
            initialdir=initialdir, filetypes=filetypes
        )

        if self.current_file == "" or self.current_file == ():
            return

        for file in self.current_file:
            self.RecentFileManager.add_file((file, type))
            self.RecentFileManager.add_files_to_menu()
            self.filenames += [(file, type)]
        self.show_preview()
        self.filemenu.entryconfig(0, state=tk.DISABLED)
        self.filemenu.entryconfig(1, state=tk.DISABLED)
        self.filemenu.entryconfig(4, state=tk.NORMAL)
        self.filemenu.entryconfig(5, state=tk.NORMAL)
        if len(self.filenames) > 2:
            self.normmenu.entryconfig(13, state=tk.DISABLED)
            self.normmenu.entryconfig(14, state=tk.DISABLED)
            self.normmenu.entryconfig(15, state=tk.DISABLED)
        self.saved = False

        return

    def load_dropped_file(self, files):
        """Loads measurement or simulation data to be displayed
        """

        for i, character in enumerate(files):
            if character == " ":
                if files[i + 1] == "{":
                    files = files[:i] + ";" + files[i + 1 :]
                else:
                    if files[:i].count("{") == files[:i].count("}"):
                        files = files[:i] + ";" + files[i + 1 :]

        files = files.split(";")
        for i, file in enumerate(files):
            if "{" in file:
                files[i] = file[1:-1]

        types = {
            "csv": "simulation",
            "3ddose": "egs",
            "bin": "simulation",
            "mcc": "ptw",
            "txt": "measurement",
        }
        extensions = []
        for i, file in enumerate(files):
            try:
                if file.split(".")[-1] == "csv":
                    with open(file) as testfile:
                        content = testfile.readlines()[0]
                        if "TOPAS" in content:
                            extensions += ["simulation"]
                        else:
                            extensions += ["measurement"]
                else:
                    extensions += [types[file.split(".")[-1]]]
            except KeyError:
                files.pop(i)

        self.current_file = files

        for i, file in enumerate(self.current_file):
            self.RecentFileManager.add_file((file, extensions[i]))
            self.RecentFileManager.add_files_to_menu()
            self.filenames += [(file, extensions[i])]

        if len(self.filenames) == 0:
            return

        self.show_preview()
        self.filemenu.entryconfig(0, state=tk.DISABLED)
        self.filemenu.entryconfig(1, state=tk.DISABLED)
        self.filemenu.entryconfig(4, state=tk.NORMAL)
        self.filemenu.entryconfig(5, state=tk.NORMAL)
        if len(self.filenames) > 2:
            self.normmenu.entryconfig(13, state=tk.DISABLED)
            self.normmenu.entryconfig(14, state=tk.DISABLED)
            self.normmenu.entryconfig(15, state=tk.DISABLED)
        self.saved = False

    def close_file(self, event=None):

        """Closes the current project
        """
        self.canvas.pack_forget()
        self.filemenu.entryconfig(0, state=tk.NORMAL)
        self.filemenu.entryconfig(1, state=tk.NORMAL)
        self.filemenu.entryconfig(4, state=tk.DISABLED)
        self.filemenu.entryconfig(5, state=tk.DISABLED)
        self.addmeasuremenu.entryconfig(0, state=tk.NORMAL)
        self.addmeasuremenu.entryconfig(1, state=tk.NORMAL)
        self.normmenu.entryconfig(13, state=tk.DISABLED)
        self.normmenu.entryconfig(14, state=tk.DISABLED)
        self.normmenu.entryconfig(15, state=tk.DISABLED)
        self.parammenu.entryconfig(0, state=tk.NORMAL)
        self.menubar.delete(4, 5)
        self.filenames = []
        try:
            self.canvas.itemconfig(self.image_on_canvas, image=None)
        except Exception:
            pass
        self.DoseFigureHandler.plots = []
        self.diffplot.set(False)
        self.DoseFigureHandler.diffplot = False
        self.DoseFigureHandler.xaxisname == None
        self.direction = None
        self.errlimmin = 1.1
        self.errlimmax = 1.1
        self.saved = True
        self.menuflag = False
        self.current_limits = []
        self.initial_limits = []
        self.rename_boxes = []
        self.slidervars = [tk.DoubleVar(), tk.DoubleVar()]
        self.DoseFigureHandler.caxcorrection = False
        self.caxcorrection.set(False)
        try:
            self.slider.window.destroy()
            self.xlimmenu = False
        except AttributeError:
            pass
        try:
            self.table.pack_forget()
            self.tablevar.set(False)
        except AttributeError:
            pass
        deltax = self.winfo_width() // 2 - self.center[0]
        deltay = self.winfo_height() // 2 - self.center[1]
        self.center = [self.winfo_width() // 2, self.winfo_height() // 2]
        self.logocanvas.move(self.logo_on_canvas, deltax, deltay)
        self.logocanvas.pack(side=tk.TOP, fill="both", expand=True)
        return

    def save_graph(self, exit=False):

        """Saves the current graph as an image
        """

        try:
            self.slider.submit()
            self.slider.window.destroy()
            self.slider = None
        except Exception:
            pass

        if exit == True:
            self.profile.set_attribute("state", self.parent.state())
            if self.parent.state() == "zoomed":
                self.profile.set_attribute("geometry", " ")
            else:
                self.profile.set_attribute("geometry", self.parent.geometry())

            if self.saved == False:

                prompt = sd.messagebox.askyesno("", self.text.closeprompt[self.lang])
                if prompt == True:
                    self.parent.destroy()
                    return
                else:
                    return
            else:
                self.parent.destroy()
                return

        file = fd.asksaveasfilename(
            defaultextension=".png",
            filetypes=[
                (self.text.image[self.lang], [".png", ".jpg"]),
                ("PDF", [".pdf"]),
            ],
        )

        if file is None:
            return

        ImageTk.getimage(self.photoimage).convert("RGB").save(file)
        self.saved = True

        return

    def normalize(self):

        """Choose whether or not the displayed graph should be normalized
        """

        self.DoseFigureHandler.norm = self.norm.get()

        self.profile.set_attribute("normalize", int(self.norm.get()))
        self.refresh()

        return

    def show_errorbars(self):

        """Choose whether or not the displayed graph should be contain errorbars
        """

        self.DoseFigureHandler.errorbars = self.errorbars.get()

        self.refresh()

        return

    def correct_caxdev(self):

        """Choose whether or not the displayed dose
        profiles should be center-axis-corrected
        """

        self.DoseFigureHandler.caxcorrection = self.caxcorrection.get()

        self.refresh()

        return

    def calculate_parameters(self):

        """Choose whether or not the displayed dose
        profiles should be center-axis-corrected
        """

        self.DoseFigureHandler.calcparams = self.calcparams.get()

        self.refresh()

        return

    def show_table(self):

        self.calcparams.set(False)
        if self.tablevar.get() == False:
            self.table.destroy()
            del self.table
        else:
            self.add_table()

        self.refresh()

        return

    def remove_last_addition(self, event=None):

        """
        Removes the graph added last
        """

        if len(self.filenames) == 1:
            self.close_file()
            return

        if len(self.filenames) == 2:
            self.diffplot.set(False)
            self.DoseFigureHandler.diffplot = False
            self.normmenu.entryconfig(13, state=tk.DISABLED)
            self.normmenu.entryconfig(14, state=tk.DISABLED)
            self.normmenu.entryconfig(15, state=tk.DISABLED)

        if self.filenames[-1][0].endswith(".mcc") == True:
            self.filenames.pop(-1)
            while len(self.filenames) < len(self.DoseFigureHandler.plots):
                self.DoseFigureHandler.plots.pop(-1)

        else:
            self.filenames.pop(-1)
            self.DoseFigureHandler.plots.pop(-1)

        if len(self.filenames) <= 4:
            self.parammenu.entryconfig(0, state=tk.NORMAL)

        self.canvas.itemconfig(self.image_on_canvas, image=None)
        self.show_preview()

        return

    def change_normalization(self):

        """Change the normalization to different points
        """

        if self.normvaluemenu.get() == "max":
            self.DoseFigureHandler.normvalue = "max"
        elif self.normvaluemenu.get() == "plateau":
            self.DoseFigureHandler.normvalue = "plateau"
        else:
            self.DoseFigureHandler.normvalue = "center"

        self.refresh()

        return

    def differenceplot(self):

        self.DoseFigureHandler.diffplot = self.diffplot.get()
        if self.diffplot.get() == True:
            self.normmenu.entryconfig(14, state=tk.NORMAL)
        else:
            self.normmenu.entryconfig(14, state=tk.DISABLED)

        self.refresh()

        return

    def change_errlims(self, boolean1, boolean2):
        if boolean1 == True:
            if boolean2 == True:
                self.DoseFigureHandler.errlimmax += 0.5
            else:
                self.DoseFigureHandler.errlimmax -= 0.5
        else:
            if boolean2 == True:
                self.DoseFigureHandler.errlimmin -= 0.5
            else:
                self.DoseFigureHandler.errlimmin += 0.5

        self.refresh()

    def changeerrdisplay(self):

        self.prev_difference = self.DoseFigureHandler.difference

        if self.errlimval.get() == "percentage":
            self.DoseFigureHandler.errlimval = "percentage"
            self.DoseFigureHandler.errlimmax = 1.1
        else:
            self.DoseFigureHandler.errlimval = "absolute"

        self.refresh()

    def zoomgraph(self):

        """Choose whether or not the displayed graph
        should contain inset axis for a zoom view
        """

        self.DoseFigureHandler.zoom = self.zoom.get()

        self.profile.set_attribute("zoom", int(self.zoom.get()))

        self.refresh()

        return

    def halfgraph(self):

        """Choose whether or not to only show half of the dose profile
        """

        self.DoseFigureHandler.half = self.half.get()
        self.profile.set_attribute("halfview", self.half.get())
        self.show_preview()

        return

    def change_marker_size(self, boolean, event=None):

        """Change the size of the matplotlib markers in the graph
        """

        if boolean == True:
            self.DoseFigureHandler.markersize += 0.2
        else:
            self.DoseFigureHandler.markersize -= 0.2

        if self.DoseFigureHandler.markersize < 0:
            self.DoseFigureHandler.markersize = 0

        self.profile.set_attribute("markersize", self.DoseFigureHandler.markersize)

        self.refresh()

        return

    def change_line_width(self, boolean, event=None):

        """Change the width of the matplotlib lines in the graph
        """

        if boolean == True:
            self.DoseFigureHandler.linewidth += 0.2
        else:
            self.DoseFigureHandler.linewidth -= 0.2
        if self.DoseFigureHandler.linewidth < 0.2:
            self.DoseFigureHandler.linewidth = 0.2

        self.profile.set_attribute("linewidth", self.DoseFigureHandler.linewidth)

        self.refresh()
        return

    def reset_colors(self):
        self.profile.set_attribute(
            "colors", self.profile.get_attribute("default_colors")
        )
        self.DoseFigureHandler.colors = self.profile.get_attribute("default_colors")
        self.show_preview()

    def reset_markers(self):
        self.profile.set_attribute(
            "markers", self.profile.get_attribute("default_markers")
        )
        self.DoseFigureHandler.colors = self.profile.get_attribute("default_markers")
        self.show_preview()

    def show_preview(self):

        """
        Invokes DoseFigureHandler to create and display the graphs with the selected options
        """
        # try:
        self.photoimage, self.direction = self.DoseFigureHandler.return_figure(
            self.filenames
        )
        # except Exception:
        #    return

        self.logocanvas.pack_forget()
        if self.tablevar.get() == True:
            if self.direction == "Z":
                if len(self.DoseFigureHandler.plots) == len(self.table._widgets[0]) - 1:
                    pass
                else:
                    self.table.destroy()
                    self.show_table()
            else:
                if len(self.DoseFigureHandler.plots) == len(self.table._widgets) - 1:
                    pass
                else:
                    self.table.destroy()
                    self.add_table()
        self.canvas.pack_forget()
        self.canvas = tk.Canvas(self)
        self.canvas.bind("<Button-1>", self.check_click)
        self.canvas.bind("<Button-3>", self.check_right_click)
        self.canvas.bind("<Motion>", self.check_hand)
        self.canvas.bind("<Double-Button-1>", self.new_zoom)
        self.parent.bind("<Up>", lambda boolean: self.change_order(True))
        self.parent.bind("<Down>", lambda boolean: self.change_order(False))

        if len(self.DoseFigureHandler.plots) >= 2:
            self.normmenu.entryconfig(13, state=tk.DISABLED)
            self.normmenu.entryconfig(14, state=tk.DISABLED)
            self.normmenu.entryconfig(15, state=tk.DISABLED)

        if len(self.DoseFigureHandler.plots) == 2:
            self.normmenu.entryconfig(13, state=tk.NORMAL)
            self.normmenu.entryconfig(14, state=tk.NORMAL)
            self.normmenu.entryconfig(15, state=tk.NORMAL)

        if len(self.DoseFigureHandler.plots) > 5:
            self.parammenu.entryconfig(0, state=tk.DISABLED)

        if self.direction == "Z":
            self.addmeasuremenu.entryconfig(1, state=tk.DISABLED)
            self.normmenu.entryconfig(8, state=tk.DISABLED)
            self.normmenu.entryconfig(11, state=tk.DISABLED)
        else:
            self.addmeasuremenu.entryconfig(0, state=tk.DISABLED)
            self.normmenu.entryconfig(8, state=tk.NORMAL)
            self.normmenu.entryconfig(11, state=tk.NORMAL)

        if self.menuflag == False:
            self.menubar.add_cascade(label=self.text.add[self.lang], menu=self.addmenu)
            self.menubar.add_cascade(label="Graph", menu=self.normmenu)
            self.menuflag = True
        self.image_on_canvas = self.canvas.create_image(
            self.winfo_width() // 2,
            self.winfo_height() // 2,
            anchor=tk.CENTER,
            image=self.photoimage,
        )

        self.canvas.image = self.photoimage
        self.canvas.itemconfig(self.image_on_canvas, image=self.canvas.image)
        self.canvas.pack(side=tk.TOP, fill="both", expand=True)

        for i in range(len(self.DoseFigureHandler.plots)):
            normdiff = 0
            if self.norm.get() == True:
                normdiff = 0.035
            factor = (self.canvas.image.width() / self.canvas.image.height()) / 2
            temp = self.canvas.create_rectangle(
                (
                    -0.00833
                    * max(
                        map(
                            len,
                            [plot.filename for plot in self.DoseFigureHandler.plots],
                        )
                    )
                    + 0.94633
                )
                * self.canvas.image.width(),
                (0.023 + normdiff + 0.042 * i) * self.canvas.image.height() * factor,
                0.988 * self.canvas.image.width(),
                (0.065 + normdiff + 0.042 * i) * self.canvas.image.height() * factor,
                fill="red",
                outline="",
                tags="rename",
            )
            self.rename_boxes += [temp]

        if self.zoom.get() == True:

            self.pixelx = self.DoseFigureHandler.pixelx
            self.pixely = self.DoseFigureHandler.pixely

            self.oval = self.canvas.create_oval(
                self.pixelx * self.canvas.image.width() - 5,
                self.pixely * self.canvas.image.height() - 5,
                self.pixelx * self.canvas.image.width() + 5,
                self.pixely * self.canvas.image.height() + 5,
                fill="",
                outline="",
                tags="token",
            )

    def add_table(self):
        try:
            if len(self.DoseFigureHandler.plots) == len(self.table._widgets[0]) - 1:
                return
        except AttributeError:
            pass
        try:

            self.fontsize = int(
                round(
                    9.333
                    + 6.6667
                    * (self.parent.winfo_width() / self.parent.winfo_screenwidth()),
                    0,
                )
            )

            if self.direction == "Z":
                self.table = SimpleTable(
                    self.parent, 4, len(self.DoseFigureHandler.plots) + 1
                )
                self.table.set(0, 0, self.text.metric[self.lang], self.fontsize)
                self.table.set(1, 0, "Q", self.fontsize)
                self.table.set(2, 0, "dQ", self.fontsize)
                self.table.set(3, 0, "z_max", self.fontsize)
                for i in range(1, len(self.DoseFigureHandler.plots) + 1):
                    self.table.set(0, i, "⬤", self.fontsize)
                    self.table._widgets[0][i].configure(
                        fg=self.DoseFigureHandler.colors[i - 1]
                    )
                for i in range(1, len(self.DoseFigureHandler.plots) + 1):
                    for j in range(1, 4):
                        self.table.set(
                            j,
                            i,
                            self.DoseFigureHandler.plots[i - 1].params()[j - 1],
                            self.fontsize,
                        )
                self.table.pack(side="bottom", fill="y", pady=(10, 2))
            else:
                self.table = SimpleTable(
                    self.parent, len(self.DoseFigureHandler.plots) + 1, 10
                )
                self.table.set(0, 0, self.text.metric[self.lang], self.fontsize)
                self.table.set(0, 1, self.text.fwhm[self.lang], self.fontsize)
                self.table.set(0, 2, "CAXdev", self.fontsize)
                self.table.set(0, 3, "flat_krieger", self.fontsize)
                self.table.set(0, 4, "flat_stddev", self.fontsize)
                self.table.set(0, 5, "S", self.fontsize)
                self.table.set(0, 6, "Lpenumbra", self.fontsize)
                self.table.set(0, 7, "Rpenumbra", self.fontsize)
                self.table.set(0, 8, "Lintegral", self.fontsize)
                self.table.set(0, 9, "Rintegral", self.fontsize)

                for i in range(1, len(self.DoseFigureHandler.plots) + 1):
                    self.table.set(i, 0, "⬤", self.fontsize)
                    self.table._widgets[i][0].configure(
                        fg=self.DoseFigureHandler.colors[i - 1]
                    )
                for i in range(1, len(self.DoseFigureHandler.plots) + 1):
                    for j in range(1, 10):
                        self.table.set(
                            i,
                            j,
                            self.DoseFigureHandler.plots[i - 1].params()[j - 1],
                            self.fontsize,
                        )
                        if self.caxcorrection.get() == True:
                            if j == 2:
                                self.table.set(
                                    i, j, "0.00", self.fontsize,
                                )

                self.table.pack(side="bottom", fill="both", padx=(2, 2), pady=(10, 2))
            self.table.configure(bg="black")
        except Exception:
            self.tablevar.set(False)
            sd.messagebox.showwarning("", self.text.calcfail[self.lang])

    def gamma(self):

        self.DoseFigureHandler.gamma()

    def changegamma(self):

        Gamma = ChangeGamma(self)

    def new_zoom(self, event):

        if self.zoom.get() == True:

            delta_x = event.x - self.canvas.coords(self.oval)[0] + 5
            delta_y = event.y - self.canvas.coords(self.oval)[1] + 5
            self.canvas.move(self.oval, delta_x, delta_y)
            self.show_preview()

    def change_order(self, boolean):

        if self.index != None:

            x = self.parent.winfo_pointerx()
            y = self.parent.winfo_pointery()

            factor = 1
            if self.DoseFigureHandler.plots[0].direction == "Z":
                factor = 6 / 5

            if boolean == True:

                new_index = self.index - 1
                dy = -0.035 * factor

                if self.index == 0:
                    new_index = len(self.rename_boxes) - 2
                    dy = 0.035 * factor * (len(self.rename_boxes) - 2)

            else:
                if self.index + 1 == len(self.rename_boxes) - 1:
                    new_index = 0
                    dy = -0.035 * factor * (len(self.rename_boxes) - 2)

                else:
                    new_index = self.index + 1
                    dy = 0.035 * factor

            (
                self.DoseFigureHandler.plots[self.index],
                self.DoseFigureHandler.plots[new_index],
            ) = (
                self.DoseFigureHandler.plots[new_index],
                self.DoseFigureHandler.plots[self.index],
            )
            (
                self.DoseFigureHandler.colors[self.index],
                self.DoseFigureHandler.colors[new_index],
            ) = (
                self.DoseFigureHandler.colors[new_index],
                self.DoseFigureHandler.colors[self.index],
            )
            self.profile.set_attribute("colors", self.DoseFigureHandler.colors)

            try:
                (self.filenames[self.index], self.filenames[new_index],) = (
                    self.filenames[new_index],
                    self.filenames[self.index],
                )
            except IndexError:
                pass

            factor = (
                1000 * self.parent.winfo_height() / self.parent.winfo_screenheight()
            )

            x = self.parent.winfo_pointerx()
            y = int(self.parent.winfo_pointery())
            y += dy * factor
            pynput.mouse.Controller().position = (x, y)
            self.index = new_index
            self.show_preview()
            self.canvas.config(cursor="hand1")
            return

    def refresh(self):

        if self.filenames != []:
            # self.canvas.itemconfig(self.image_on_canvas, image=None)
            self.show_preview()
        return

    def handle_configure(self, event=None):

        """
        Dynamically resizes the graph, rename boxes and zoom window selector
        according to the window size and name length
        """

        state = self.profile.get_attribute("state")
        if state != self.parent.state():
            self.profile.set_attribute("state", self.parent.state())
            self.starttime -= 0.1
        if type(event.widget) == tk.Label:

            return

        deltax = self.winfo_width() // 2 - self.center[0]
        deltay = self.winfo_height() // 2 - self.center[1]
        self.center = [self.winfo_width() // 2, self.winfo_height() // 2]

        self.logocanvas.move(self.logo_on_canvas, deltax, deltay)

        if self.tablevar.get() == True:

            fontsize = int(
                round(
                    9.333
                    + 6.6667
                    * (self.parent.winfo_width() / self.parent.winfo_screenwidth()),
                    0,
                )
            )
            if fontsize != self.fontsize:
                for label in self.table.winfo_children():
                    self.fontsize = fontsize
                    if isinstance(label, tk.Label):
                        label.configure(
                            text=label.cget("text"), font=("DejaVu Sans", fontsize)
                        )

        if len(self.rename_boxes) > 0:

            if time.time() > self.starttime:
                try:
                    image_width = self.winfo_width()
                    image_height = self.winfo_height()
                    photoimage_width = self.photoimage.width()
                    photoimage_height = self.photoimage.height()
                    width_factor = image_width / photoimage_width
                    height_factor = image_height / photoimage_height

                    if abs(width_factor - 1) <= 0.01 and abs(height_factor - 1) < 0.01:
                        return

                    if width_factor <= height_factor:
                        scale_factor = width_factor
                    else:
                        scale_factor = height_factor

                    self.resizable_image = ImageTk.getimage(self.photoimage)
                    self.resized_image = self.resizable_image.resize(
                        (
                            int(self.photoimage.width() * scale_factor),
                            int(self.photoimage.height() * scale_factor),
                        ),
                        Image.ANTIALIAS,
                    )
                    self.canvas.pack_forget()
                    newimage = ImageTk.PhotoImage(self.resized_image)
                    self.canvas.image = newimage
                    self.canvas.itemconfig(
                        self.image_on_canvas, image=self.canvas.image
                    )
                    self.canvas.pack(side=tk.TOP, fill="both", expand=True)
                except AttributeError:
                    pass

            coords = self.canvas.coords(self.image_on_canvas)
            imdims = [self.canvas.image.width(), self.canvas.image.height()]
            canvdims = [self.canvas.winfo_width(), self.canvas.winfo_height()]

            temp = self.rename_boxes[: len(self.DoseFigureHandler.plots)]
            if len(self.rename_boxes) == 1:
                temp = self.rename_boxes

            self.canvas.delete("rename")
            self.rename_boxes = []
            for i in range(len(temp)):
                normdiff = 0
                factor = 1
                if self.norm.get() == False:
                    normdiff = 0.025
                if self.DoseFigureHandler.plots[0].direction == "Z":
                    factor = 6 / 5

                if self.DoseFigureHandler.diffplot == True:
                    factor *= 0.75

                self.rename_boxes += [
                    self.canvas.create_rectangle(
                        coords[0]
                        + imdims[0]
                        * (
                            0.438
                            - 0.0082
                            * max(
                                map(
                                    len,
                                    [
                                        plot.filename
                                        for plot in self.DoseFigureHandler.plots
                                    ],
                                )
                            )
                        ),
                        (
                            (canvdims[1] - imdims[1]) // 2
                            + imdims[1] * (0.02 + i * 0.0347 + normdiff)
                        )
                        * factor,
                        coords[0] + imdims[0] * 0.488,
                        (
                            (canvdims[1] - imdims[1]) // 2
                            + imdims[1] * (0.0547 + i * 0.0347 + normdiff)
                        )
                        * factor,
                        fill="",
                        outline="",
                        tags="rename",
                    )
                ]

            self.canvas.delete("token")

            if self.zoom.get() == True:
                self.oval = self.canvas.create_oval(
                    self.pixelx * (self.canvas.image.width())
                    - 5
                    + (canvdims[0] - imdims[0]) // 2,
                    self.pixely * (self.canvas.image.height())
                    + (canvdims[1] - imdims[1]) // 2
                    - 5,
                    self.pixelx * (self.canvas.image.width())
                    + 5
                    + (canvdims[0] - imdims[0]) // 2,
                    self.pixely * (self.canvas.image.height())
                    + 5
                    + (canvdims[1] - imdims[1]) // 2,
                    fill="",
                    outline="",
                    tags="token",
                )

            self.canvas.delete("xaxis")
            if len(self.DoseFigureHandler.plots) >= 1:
                coords = self.canvas.coords(self.image_on_canvas)
                xbox = self.canvas.create_rectangle(
                    coords[0] - imdims[0] * (0.035 + normdiff / 4),
                    imdims[1] * 0.95 + (canvdims[1] - imdims[1]) // 2,
                    coords[0] + imdims[0] * (0.065 - normdiff / 4),
                    imdims[1] + (canvdims[1] - imdims[1]) // 2,
                    tags="xaxis",
                    outline="",
                    fill="",
                )
                self.rename_boxes += [xbox]

            self.canvas.move(self.image_on_canvas, deltax, deltay)

        self.starttime = time.time()

    def check_hand(self, e):

        """Shows a different curson when hovering over a rename box
        """

        if e != None:
            try:
                hoverlist = []
                for box in self.rename_boxes:
                    bbox = self.canvas.bbox(box)
                    if (
                        bbox[0] < e.x
                        and bbox[2] > e.x
                        and bbox[1] < e.y
                        and bbox[3] > e.y
                    ):
                        hoverlist += [True]
                    else:
                        hoverlist += [False]
            except TypeError:
                self.show_preview()

            if True in hoverlist:
                self.canvas.config(cursor="hand1")
            else:
                self.canvas.config(cursor="")

            try:
                self.index = hoverlist.index(True)
            except Exception as e:
                self.index = None

    def check_click(self, e):

        """Renames a plot when the associated box is clicked
        """

        for index, box in enumerate(self.rename_boxes):
            if e != None:
                bbox = self.canvas.bbox(box)
                if bbox != None:
                    if (
                        bbox[0] < e.x
                        and bbox[2] > e.x
                        and bbox[1] < e.y
                        and bbox[3] > e.y
                    ):
                        if index == len(self.rename_boxes) - 1:
                            if self.xlimmenu == False:
                                self.change_xrange()
                            else:
                                dialog = GraphNameAndStyle(self, "X")
                                self.parent.wait_window(dialog.top)

                                self.show_preview()
                            return
                        dialog = GraphNameAndStyle(self, index)
                        self.parent.wait_window(dialog.top)

                        self.show_preview()

    def check_right_click(self, e):

        """Reassigns a color to a plot when the associated box is right-clicked
        """

        for index, box in enumerate(self.rename_boxes[:-1]):
            if e != None:
                bbox = self.canvas.bbox(box)
                if bbox != None:
                    if (
                        bbox[0] < e.x
                        and bbox[2] > e.x
                        and bbox[1] < e.y
                        and bbox[3] > e.y
                    ):
                        newcolor = askcolor(color=self.DoseFigureHandler.colors[index])
                        if newcolor != (None, None):
                            self.DoseFigureHandler.colors[index] = newcolor[1]
                            self.profile.set_attribute(
                                "colors", self.DoseFigureHandler.colors
                            )
                            self.show_preview()




import os
import tkinter as tk
from collections import deque

from matplotlib.pyplot import axes

from ..resources.language import Text
from .profile import ProfileHandler


class RecentFileManager:
    def __init__(self, parent):

        self.parent = parent

        self.recent_files = self.parent.profile.get_attribute("recent_files")

        pop = []
        for i, x in enumerate([file[0] for file in self.recent_files]):
            if os.path.exists(x) == True:
                pop.append(i)

        self.recent_files = [self.recent_files[i] for i in pop]

        self.lambdas = [
            lambda: self.add_recent_file_to_data(0),
            lambda: self.add_recent_file_to_data(1),
            lambda: self.add_recent_file_to_data(2),
            lambda: self.add_recent_file_to_data(3),
            lambda: self.add_recent_file_to_data(4),
        ]

    def add_file(self, file):

        if file[0] in [file[0] for file in self.recent_files]:
            self.recent_files.pop(
                [file[0] for file in self.recent_files].index(file[0])
            )
            self.recent_files.append(None)
            rotatable = deque(self.recent_files)
            rotatable.rotate(1)
            self.recent_files = list(rotatable)
            self.recent_files[0] = file
            self.parent.profile.set_attribute("recent_files", self.recent_files)

        elif len(self.recent_files) == 5:
            self.recent_files.pop(-1)
            self.recent_files.append(None)
            rotatable = deque(self.recent_files)
            rotatable.rotate(1)
            self.recent_files = list(rotatable)
            self.recent_files[0] = file
            self.parent.profile.set_attribute("recent_files", self.recent_files)

        else:
            self.recent_files.append(file)
            rotatable = deque(self.recent_files)
            rotatable.rotate(1)
            self.recent_files = list(rotatable)
            self.parent.profile.set_attribute("recent_files", self.recent_files)

    def add_recent_file_to_data(self, index):
        if self.recent_files[index][0] in [file[0] for file in self.parent.filenames]:
            return
        if os.path.exists(self.recent_files[index][0]) == True:

            self.parent.load_dropped_file("{" + self.recent_files[index][0] + "}")
            self.add_files_to_menu()

        else:
            self.recent_files.pop(index)
            self.recentmenu.delete(index)

    def add_files_to_menu(self):
        try:
            self.recentmenu.destroy()
        except AttributeError:
            pass

        self.recentmenu = tk.Menu(self.parent.menubar, tearoff=False)

        if len(self.recent_files) == 0:
            self.recentmenu.add_command(label="---")
            self.recentmenu.entryconfig(0, state=tk.DISABLED)

        else:

            for index, file in enumerate(self.recent_files):

                self.recentmenu.add_command(
                    label=os.path.basename(file[0]), command=self.lambdas[index]
                )

        self.parent.filemenu.insert_cascade(
            2,
            label=Text().recent[ProfileHandler().get_attribute("language")],
            menu=self.recentmenu,
        )
        self.parent.filemenu.delete(3)




import time
import tkinter as tk
import tkinter.ttk as ttk

from ..resources.language import Text
from .profile import ProfileHandler
from .RangeSlider import RangeSliderH


class XRangeSlider:
    def __init__(self, parent, slidervars, initial_limits):

        self.parent = parent
        self.window = tk.Toplevel()
        self.window.resizable(False, False)
        self.window.wm_overrideredirect(True)
        self.window.wm_attributes("-alpha", 0.6)
        self.window.bind("<Return>", self.submit)
        self.dark = self.parent.dark.get()
        self.geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]
        self.height = 110
        self.window.geometry(
            f"{500}x{self.height}+{int(self.geometry[0]+self.geometry[2]//2) - 250}+{int(self.geometry[1]+self.geometry[3]//2)}"
        )

        self.slider = RangeSliderH(
            self.window,
            variables=slidervars,
            padX=100,
            valueSide="BOTTOM",
            Width=500,
            min_val=initial_limits[0],
            max_val=initial_limits[1],
            font_family="Helvetica",
            font_size=12,
            digit_precision=".0f",
            suffix=" mm",
        )
        if self.dark == True:
            self.slider.canv.configure(bg="#363636")
            self.slider.canv.itemconfigure(self.slider.bars[0]["Ids"][2], fill="white")
            self.slider.canv.itemconfigure(self.slider.bars[1]["Ids"][2], fill="white")

        self.submitbutton = ttk.Button(
            self.window,
            text=Text().submit[ProfileHandler().get_attribute("language")],
            command=self.submit,
        )
        self.resetbutton = ttk.Button(
            self.window,
            text=Text().reset[ProfileHandler().get_attribute("language")],
            command=self.reset,
        )
        self.spacer = tk.Label(self.window, text="")
        self.submitbutton.grid(row=0, column=3, sticky="NW")
        self.resetbutton.grid(row=0, column=1, sticky="NE")
        self.spacer.grid(row=1, column=2)
        self.slider.grid(row=2, column=0, columnspan=5, sticky="S")

        self.a = slidervars[0].trace("w", self.update)
        self.b = slidervars[1].trace("w", self.update)
        self.stay_on_top()

        self.starttime = time.time()

    def update(self, *args):

        if time.time() > self.starttime + 0.1:
            # self.parent.canvas.pack_forget()
            self.parent.refresh()
            # self.parent.canvas.pack(side=tk.TOP, fill="both", expand=True)
            self.starttime = time.time()
        return

    def submit(self, event=None):
        self.window.unbind("<Return>")
        self.parent.slidervars[0].trace_vdelete("w", self.a)
        self.parent.slidervars[1].trace_vdelete("w", self.b)
        self.parent.xlimmenu = False
        self.parent.parent.bind("<MouseWheel>", self.parent.change_x_limits)
        self.window.destroy()
        self.parent.refresh()
        return

    def reset(self):
        self.parent.current_limits = self.parent.initial_limits
        self.parent.slidervars[0].set(self.parent.initial_limits[0])
        self.parent.slidervars[1].set(self.parent.initial_limits[1])
        self.slider.destroy()
        self.slider = None
        self.slider = RangeSliderH(
            self.window,
            variables=self.parent.slidervars,
            padX=100,
            valueSide="BOTTOM",
            Width=500,
            min_val=self.parent.current_limits[0],
            max_val=self.parent.current_limits[1],
            font_family="Helvetica",
            font_size=12,
            digit_precision=".0f",
            suffix=" mm",
        )
        self.slider.grid(row=1, column=0, columnspan=2, sticky="N")
        self.submit()
        return

    def stay_on_top(self):

        if self.parent.dark.get() != self.dark:
            self.dark = self.parent.dark.get()
            if self.dark == True:
                self.slider.canv.configure(bg="#363636")
                self.slider.canv.itemconfigure(
                    self.slider.bars[0]["Ids"][2], fill="white"
                )
                self.slider.canv.itemconfigure(
                    self.slider.bars[1]["Ids"][2], fill="white"
                )
            else:
                self.slider.canv.configure(bg="#ffffff")
                self.slider.canv.itemconfigure(
                    self.slider.bars[0]["Ids"][2], fill="black"
                )
                self.slider.canv.itemconfigure(
                    self.slider.bars[1]["Ids"][2], fill="black"
                )

        self.new_geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]
        self.window.lift()
        self.window.after(50, self.stay_on_top)
        if self.new_geometry == self.geometry:
            return

        self.geometry = self.new_geometry
        self.window.geometry(
            f"{500}x{self.height}+{int(self.geometry[0]+self.geometry[2]//2) - 250}+{int(self.geometry[1]+self.geometry[3]//2)}"
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 16 12:47:50 2021

@author: Sebastian Schäfer
@institution: Martin-Luther-Universität Halle-Wittenberg
@email: sebastian.schaefer@student.uni-halle.de
"""

import os
import sys
import tkinter as tk
import tkinter.ttk as ttk

from .src.classes.install_dnd import InstallDnD
from .src.classes.main_viewer import MainApplication
from .src.classes.profile import ProfileHandler
from .src.classes.update import CheckForUpdates


def topasgraphsim():

    if ProfileHandler().get_attribute("draganddrop") == True:

        try:
            import TkinterDnD2 as dnd
        except ImportError:

            drag = InstallDnD()
            print(drag.message)

            if drag.install_success == True:
                python = sys.executable
                args = "-m topasgraphsim"
                os.execl(python, python, args)
                return

        if "TkinterDnD2" in sys.modules.keys():
            try:
                root = dnd.TkinterDnD.Tk()
            except RuntimeError:
                ProfileHandler().set_attribute("draganddrop", False)
                python = sys.executable
                args = args = "-m topasgraphsim"
                os.execl(python, python, args)
                return
        else:
            ProfileHandler().set_attribute("draganddrop", False)
            python = sys.executable
            os.execl(python, python, *sys.argv)
            return
    else:
        root = tk.Tk()

    screen_width = root.winfo_screenwidth()
    screen_height = root.winfo_screenheight()
    width = screen_width // 2
    height = screen_height // 2
    root.minsize(width, height - 100)

    try:
        root.geometry(ProfileHandler().get_attribute("geometry"))
    except Exception:
        x = screen_width // 2 - width // 2
        y = screen_height // 2 - height // 2
        root.geometry(f"{width}x{height}+{x-25}+{y}")
    try:
        root.state(ProfileHandler().get_attribute("state"))
    except Exception:
        root.state("normal")

    ttk.Style(root)
    root.tk.call(
        "source",
        str(
            os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "src",
                "Azure-ttk-theme",
                "azure.tcl",
            )
        ),
    )
    root.after(
        50,
        root.iconphoto(
            True,
            tk.PhotoImage(
                file=os.path.join(
                    os.path.dirname(os.path.realpath(__file__)),
                    "src",
                    "resources",
                    "icon.png",
                ),
                master=root,
            ),
        ),
    )

    app = MainApplication(root)

    try:

        def drop_enter(event):
            event.widget.focus_force()
            return event.action

        def drop_position(event):
            return event.action

        def drop_leave(event):
            return event.action

        def drop(event):
            if event.data:
                app.load_dropped_file(event.data)
            return event.action

        root.drop_target_register(dnd.DND_FILES)
        root.dnd_bind("<<DropEnter>>", drop_enter)
        root.dnd_bind("<<DropPosition>>", drop_position)
        root.dnd_bind("<<DropLeave>>", drop_leave)
        root.dnd_bind("<<Drop>>", drop)
    except Exception:
        pass

    CheckForUpdates()

    root.mainloop()


if __name__ == "__main__":
    topasgraphsim()



import tkinter as tk
import tkinter.ttk as ttk

from ..resources.language import Text


class ChangeGamma:
    def __init__(self, parent) -> None:

        self.parent = parent
        self.gammathreshold = self.parent.gammathreshold
        self.gammadist = self.parent.gammadist

        self.top = tk.Toplevel()
        self.geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]
        self.width = 110
        self.height = 140

        self.top.geometry(
            f"{self.width}x{self.height}+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )
        self.top.overrideredirect(1)

        self.top.bind("<Return>", self.submit)
        self.top.bind("<Escape>", self.top.destroy)
        self.enterlabel = ttk.Label(self.top, text="Gamma-Index")
        self.enterbox1 = ttk.Entry(self.top, width=3)
        self.enterbox1.insert(tk.END, self.gammathreshold)
        self.percentlabel = ttk.Label(self.top, text="%")
        self.enterbox2 = ttk.Entry(self.top, width=3)
        self.enterbox2.insert(tk.END, self.gammadist)
        self.mmtabel = ttk.Label(self.top, text="mm")

        self.enterlabel.grid(row=0, column=0, columnspan=2, padx=(5, 5), pady=(2, 2))
        self.enterbox1.grid(row=1, column=0, padx=(1, 1), pady=(2, 2))
        self.percentlabel.grid(row=1, column=1, padx=(1, 1), pady=(2, 2))
        self.enterbox2.grid(row=2, column=0, padx=(1, 1), pady=(2, 2))
        self.mmtabel.grid(row=2, column=1, padx=(1, 1), pady=(2, 2))
        self.submitbutton = ttk.Button(
            self.top, text=Text().submit[self.parent.lang], command=self.submit,
        )
        self.submitbutton.grid(row=3, column=0, columnspan=2, padx=(5, 5), pady=(2, 2))

        self.enterbox1.focus()
        self.lift()

    def lift(self):
        self.new_geometry = [
            self.parent.parent.winfo_rootx(),
            self.parent.parent.winfo_rooty(),
            self.parent.winfo_width(),
            self.parent.winfo_height(),
        ]

        self.top.lift()
        self.top.after(100, self.lift)
        if self.new_geometry == self.geometry:
            return

        self.geometry = self.new_geometry
        self.top.geometry(
            f"{self.width}x{self.height}+{self.parent.winfo_rootx()}+{self.parent.parent.winfo_rooty()}"
        )

    def submit(self, event=None):

        try:
            self.gammathreshold = float(self.enterbox1.get())
            self.gammadist = float(self.enterbox2.get())
        except Exception:
            self.top.destroy()

        if self.gammadist > 0 and self.gammathreshold > 0:

            self.parent.gammathreshold = self.gammathreshold
            self.parent.gammadist = self.gammadist
            self.parent.gammaacc.set(f"{self.gammathreshold}%/{self.gammadist}mm")
            self.parent.gammamenu.entryconfig(1, accelerator=self.parent.gammaacc.get())

        self.top.destroy()



from scipy.interpolate import CubicSpline
import numpy as np


def calculate_parameters(axis, dose, std_dev):

    """
    A function to calculate the relevant 
    descriptive parameters of depth doses.
    """

    def findnearestdatapoint(listname, value):
        a = []
        for i in range(len(listname)):
            a.append(abs(listname[i] - value))
        return a.index(min(a))

    ################# Interpolate
    cs_sim = CubicSpline(axis, dose)
    posspline = np.arange(0, 300, 0.01)
    dosespline = cs_sim(posspline)
    # Berechnung Q

    ind20 = findnearestdatapoint(posspline, 200)
    ind10 = findnearestdatapoint(posspline, 100)
    TD20 = dosespline[ind20]
    TD10 = dosespline[ind10]
    Q = round(1.2661 * TD20 / TD10 - 0.0595, 5)  # DIN 6800-2
    dQ = 0
    # Berechnung zmax
    zmax = round(posspline[np.argmax(dosespline)], 5)

    return [Q, dQ, zmax]




import tkinter as tk


class ExampleApp(tk.Tk):
    def __init__(self):
        tk.Tk.__init__(self)
        t = SimpleTable(self, 10, 2)
        t.pack(side="top", fill="x")
        t.set(0, 0, "Hello, world")


class SimpleTable(tk.Frame):
    def __init__(self, parent, rows=10, columns=2):
        # use black background so it "peeks through" to
        # form grid lines
        tk.Frame.__init__(self, parent, bg="white")
        self._widgets = []
        for row in range(rows):
            current_row = []
            for column in range(columns):
                label = tk.Label(
                    self, borderwidth=0, width=10, font=("DejaVu Sans", 13)
                )
                label.grid(
                    row=row, column=column, sticky="nsew", padx=1, pady=1,
                )
                current_row.append(label)
            self._widgets.append(current_row)

        for column in range(columns):
            self.grid_columnconfigure(column, weight=1)

    def set(self, row, column, value, fontsize):
        widget = self._widgets[row][column]
        widget.configure(text=value, font= ("DejaVu Sans", fontsize))



from tkinter import *
from tkinter.ttk import *



class RangeSliderH(Frame):
    LINE_COLOR = "#476b6b"
    LINE_S_COLOR="#0a50ff"
    LINE_WIDTH = 3
    BAR_COLOR_INNER = "#5c8a8a"
    BAR_COLOR_OUTTER = "#c2d6d6"
    BAR_RADIUS = 10
    BAR_RADIUS_INNER = 5
    DIGIT_PRECISION = '.1f' # for showing in the canvas
    FONT_SIZE=16
    FONT_FAMILY='Times'
    IMAGE_ANCHOR_L=CENTER
    IMAGE_ANCHOR_R=CENTER
    def __init__(self, master, variables, Width = 400, Height = 80, min_val = 0, max_val = 1, padX=3,
                    image_anchorR=CENTER, image_anchorL=CENTER, imageL=None, imageR=None,
                    auto=True, line_width=3, bar_radius=10,
                    bar_color_inner='#5c8a8a', line_s_color="#0a50ff", bar_color_outer='#c2d6d6', line_color = '#476b6b', bgColor= '#ffffff',
                    show_value = True, digit_precision='.1f', valueSide='TOP', font_family='Times', font_size=16, suffix=""):
        RangeSliderH.LINE_COLOR=line_color
        RangeSliderH.LINE_WIDTH=line_width
        RangeSliderH.BAR_COLOR_INNER=bar_color_inner
        RangeSliderH.BAR_COLOR_OUTTER=bar_color_outer
        RangeSliderH.BAR_RADIUS=bar_radius
        RangeSliderH.BAR_RADIUS_INNER=bar_radius-5
        RangeSliderH.DIGIT_PRECISION=digit_precision
        RangeSliderH.FONT_SIZE=font_size
        RangeSliderH.FONT_FAMILY=font_family
        RangeSliderH.IMAGE_ANCHOR_L=image_anchorL
        RangeSliderH.IMAGE_ANCHOR_R=image_anchorR
        RangeSliderH.LINE_S_COLOR=line_s_color
        if auto:
            if imageL!=None or imageR!=None:
                raise Exception("Can't decide if to use auto shape or images!")
            else:
                critical1=max(max(len(str(min_val)+suffix),len(str(max_val)+suffix))*RangeSliderH.FONT_SIZE*1.33/4,RangeSliderH.BAR_RADIUS)
                critical2=2*(padX+RangeSliderH.BAR_RADIUS)
                critical3=2*(1.33*RangeSliderH.FONT_SIZE+RangeSliderH.BAR_RADIUS)
                if show_value and padX<critical1:
                    raise Exception("padX too small, value won't be visible completely, expected padX to be atleast "+ str(critical1+1)+"px.")
                if Width<critical2:
                    raise Exception("Dimensions incompatible, consider decreasing padX or bar_radius or consider increasing widget width, as per given configuration width should be atleast ."+str(critical2+1)+"px.")
                if Height<critical3:
                    raise Exception("Dimensions incompatible, consider decreasing bar_radius or consider increasing widget height, as per given configuration height should be atleast ."+str(critical3+1)+"px.")
                if RangeSliderH.BAR_RADIUS<=line_width:
                    raise Exception("bar_radius too small, should be atleast equal to line_width(default=10)")
                self.draw='auto'
        else:
            if imageL==None or imageR==None:
                raise Exception("Handle for one image missing.")
            else:
                critical1=imageL.height()+2*1.33*RangeSliderH.FONT_SIZE
                critical2=imageR.width()*2+2*padX
                critical3=max(len(str(min_val)+suffix),len(str(max_val)+suffix))*RangeSliderH.FONT_SIZE*1.33/4
                if imageL.height()==imageR.height() and imageL.width()==imageR.width():
                    if critical1<Height and critical2< Width:
                        if show_value and padX<critical3:
                            raise Exception("padX too small, value won't be visible completely, expected padX to be atleast "+str(critical3+1)+"px.")
                        self.draw='image'
                    else:
                        raise Exception("Image dimensions not suited with widget Height and width, as per given configuration height should be atleast "+str(critical1+1)+"px and width should be atleast "+str(critical2+1)+"px.")
                else:
                    raise Exception("Image dimensions incompatible, both handles should have same height and width respectively.")
        Frame.__init__(self, master, height = Height, width = Width)
        self.padx=padX
        self.ImageL=imageL
        self.ImageR=imageR
        self.master = master
        self.max_val = max_val
        self.min_val = min_val
        self.show_value = show_value
        self.valueSide=valueSide
        self.H = Height
        self.W = Width
        self.canv_H = self.H
        self.canv_W = self.W
        self.suffix=suffix
        self.variables = variables
        try:
            self.init_lis = [variables[0].get(), variables[1].get()]
        except Exception:
            self.init_lis = [min_val, max_val]
        if not show_value:
            self.slider_y = self.canv_H/2 # y pos of the slider
        else:
            if self.valueSide=='BOTTOM':
                if self.draw=='auto':
                    self.slider_y = RangeSliderH.BAR_RADIUS+2
                else:
                    self.slider_y = imageL.height()/2 + 2
            elif self.valueSide=='TOP':
                if self.draw=='auto':
                    self.slider_y = self.canv_H - RangeSliderH.BAR_RADIUS - 2
                else:
                    self.slider_y = self.canv_H - imageL.height()/2 - 2
            else:
                raise Exception("valueSide can either be TOP or BOTTOM.")
        if self.draw=='auto':
            self.slider_x = RangeSliderH.BAR_RADIUS+self.padx # x pos of the slider (left side)
        else:
            self.slider_x = self.ImageL.width()/2+self.padx

        self.bars = []
        self.selected_idx = None # current selection bar index
        for value in self.init_lis:
            pos = (value-min_val)/(max_val-min_val)
            ids = []
            bar = {"Pos":pos, "Ids":ids, "Value":value}
            self.bars.append(bar)


        self.canv = Canvas(self, height = self.canv_H, width = self.canv_W, bg=bgColor, bd=0 , highlightthickness=0, relief='ridge')
        self.canv.pack()
        self.canv.bind("<Motion>", self._mouseMotion)
        self.canv.bind("<B1-Motion>", self._moveBar)

        self.track = self.__addTrack(self.slider_x, self.slider_y, self.canv_W-self.slider_x, self.slider_y, self.bars[0]["Pos"], self.bars[1]["Pos"])
        tempIdx=0
        for bar in self.bars:
            bar["Ids"] = self.__addBar(bar["Pos"],tempIdx)
            tempIdx+=1

        self.__setValues()


    def getValues(self):
        values = [bar["Value"] for bar in self.bars]
        return values

    def __setValues(self):
        values=self.getValues()
        self.variables[0].set(values[0])
        self.variables[1].set(values[1])

    def getPos(self):
        poss = [bar["Pos"] for bar in self.bars]
        return poss

    def _mouseMotion(self, event):
        x = event.x; y = event.y
        selection = self.__checkSelection(x,y)
        if selection[0]:
            self.canv.config(cursor = "hand2")
            self.selected_idx = selection[1]
        else:
            self.canv.config(cursor = "")
            self.selected_idx = None

    def _moveBar(self, event):
        x = event.x; y = event.y
        if self.selected_idx == None:
            return False
        pos = self.__calcPos(x)
        idx = self.selected_idx
        self.__moveBar(idx,pos)

    def __addTrack(self, startx, starty, endx, endy, posL, posR):
        rangeOutL = self.canv.create_line(startx, starty, startx+posL*(endx-startx), endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        rangeS = self.canv.create_line(startx+posL*(endx-startx), starty, endx-(1-posR)*(endx-startx), endy, fill = RangeSliderH.LINE_S_COLOR, width = RangeSliderH.LINE_WIDTH)
        rangeOutR = self.canv.create_line(endx-(1-posR)*(endx-startx), starty, endx, endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        return [rangeOutL, rangeS, rangeOutR]

    def __addTrackL(self, startx, starty, endx, endy, posL, posR):
        rangeOutL = self.canv.create_line(startx, starty, startx+posL*(endx-startx), endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        rangeS = self.canv.create_line(startx+posL*(endx-startx), starty, endx-(1-posR)*(endx-startx), endy, fill = RangeSliderH.LINE_S_COLOR, width = RangeSliderH.LINE_WIDTH)
        # rangeOutR = self.canv.create_line(endx-(1-posR)*(endx-startx), starty, endx, endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        return [rangeOutL, rangeS]

    def __addTrackR(self, startx, starty, endx, endy, posL, posR):
        # rangeOutL = self.canv.create_line(startx, starty, startx+posL*(endx-startx), endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        rangeS = self.canv.create_line(startx+posL*(endx-startx), starty, endx-(1-posR)*(endx-startx), endy, fill = RangeSliderH.LINE_S_COLOR, width = RangeSliderH.LINE_WIDTH)
        rangeOutR = self.canv.create_line(endx-(1-posR)*(endx-startx), starty, endx, endy, fill = RangeSliderH.LINE_COLOR, width = RangeSliderH.LINE_WIDTH)
        return [rangeS, rangeOutR]

    def __addBar(self, pos, tempIdx=None):
        """@ pos: position of the bar, ranged from (0,1)"""
        if pos <0 or pos >1:
            raise Exception("Pos error - Pos: "+str(pos))
        if self.draw=='auto':
            R = RangeSliderH.BAR_RADIUS
            r = RangeSliderH.BAR_RADIUS_INNER
            L = self.canv_W - 2*self.slider_x
            y = self.slider_y
            x = self.slider_x+pos*L
            id_outer = self.canv.create_oval(x-R,y-R,x+R,y+R, fill = RangeSliderH.BAR_COLOR_OUTTER, width = 2, outline = "")
            id_inner = self.canv.create_oval(x-r,y-r,x+r,y+r, fill = RangeSliderH.BAR_COLOR_INNER, outline = "")
            if self.show_value:
                if self.valueSide=='TOP':
                    y_value = y-RangeSliderH.BAR_RADIUS-RangeSliderH.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x,y_value,anchor=S, text = format(value, RangeSliderH.DIGIT_PRECISION)+self.suffix,font=(RangeSliderH.FONT_FAMILY,RangeSliderH.FONT_SIZE))
                elif self.valueSide=='BOTTOM':
                    y_value = y+RangeSliderH.BAR_RADIUS+RangeSliderH.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x,y_value,anchor=N, text = format(value, RangeSliderH.DIGIT_PRECISION)+self.suffix,font=(RangeSliderH.FONT_FAMILY,RangeSliderH.FONT_SIZE))
                else:
                    raise Exception("valueSide can either be TOP or BOTTOM")
                return [id_outer, id_inner, id_value]
            else:
                return [id_outer, id_inner]
        elif self.draw=='image':
            L=self.canv_W - 2*self.slider_x
            y=self.slider_y
            x=self.slider_x+pos*L
            if tempIdx==0:
                imageH=self.canv.create_image(x,y,anchor=RangeSliderH.IMAGE_ANCHOR_L,image=self.ImageL)
            elif tempIdx==1:
                imageH=self.canv.create_image(x,y,anchor=RangeSliderH.IMAGE_ANCHOR_R,image=self.ImageR)
            if self.show_value:
                if self.valueSide=='TOP':
                    y_value = y-self.ImageL.height()/2-RangeSliderH.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x,y_value,anchor=S, text = format(value, RangeSliderH.DIGIT_PRECISION)+self.suffix,font=(RangeSliderH.FONT_FAMILY,RangeSliderH.FONT_SIZE))
                elif self.valueSide=='BOTTOM':
                    y_value = y+self.ImageL.height()/2+RangeSliderH.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x,y_value,anchor=N, text = format(value, RangeSliderH.DIGIT_PRECISION)+self.suffix,font=(RangeSliderH.FONT_FAMILY,RangeSliderH.FONT_SIZE))
                else:
                    raise Exception("valueSide can either be TOP or BOTTOM")
                return [imageH, id_value]
            else:
                return [imageH]


    def __moveBar(self, idx, pos):
        positions=self.getPos()
        current_pos=self.bars[idx]["Pos"]
        for i in range(0,2):
            ids=self.bars[i]["Ids"]
            for id in ids:
                self.canv.delete(id)
        for trackComponentsId in self.track[idx:idx+2]:
            self.canv.delete(trackComponentsId)
        if idx==0:
            otherIdx=1
            otherPos=positions[1]
            if pos<=otherPos:
                pos=pos
            else:
                pos=current_pos
            self.track[0:2] = self.__addTrackL(self.slider_x, self.slider_y, self.canv_W-self.slider_x, self.slider_y, pos, otherPos)
        else:
            otherIdx=0
            otherPos=positions[0]
            if pos>=otherPos:
                pos=pos
            else:
                pos=current_pos
            self.track[1:3] = self.__addTrackR(self.slider_x, self.slider_y, self.canv_W-self.slider_x, self.slider_y, otherPos, pos)
        
        self.bars[idx]["Ids"] = self.__addBar(pos, idx)
        self.bars[idx]["Pos"] = pos
        self.bars[idx]["Value"] = pos*(self.max_val - self.min_val)+self.min_val

        self.bars[otherIdx]["Ids"] = self.__addBar(otherPos, otherIdx)
        self.bars[otherIdx]["Pos"] = otherPos
        self.bars[otherIdx]["Value"] = otherPos*(self.max_val - self.min_val)+self.min_val

        self.__setValues()

    def __calcPos(self, x):
        """calculate position from x coordinate"""
        pos = (x - self.slider_x)/(self.canv_W-2*self.slider_x)
        if pos<0:
            return 0
        elif pos>1:
            return 1
        else:
            return pos

    def __getValue(self, idx):
        """#######Not used function#####"""
        bar = self.bars[idx]
        ids = bar["Ids"]
        x = self.canv.coords(ids[0])[0] + RangeSliderH.BAR_RADIUS
        pos = self.__calcPos(x)
        return pos*(self.max_val - self.min_val)+self.min_val

    def __checkSelection(self, x, y):
        """
        To check if the position is inside the bounding rectangle of a Bar
        Return [True, bar_index] or [False, None]
        """
        for idx in range(len(self.bars)):
            id = self.bars[idx]["Ids"][0]
            bbox = self.canv.bbox(id)
            if bbox[0] < x and bbox[2] > x and bbox[1] < y and bbox[3] > y:
                return [True, idx]
        return [False, None]


class RangeSliderV(Frame):
    LINE_COLOR = "#476b6b"
    LINE_S_COLOR="#0a50ff"
    LINE_WIDTH = 3
    BAR_COLOR_INNER = "#5c8a8a"
    BAR_COLOR_OUTTER = "#c2d6d6"
    BAR_RADIUS = 10
    BAR_RADIUS_INNER = 5
    DIGIT_PRECISION = '.1f' # for showing in the canvas
    FONT_SIZE=16
    FONT_FAMILY='Times'
    IMAGE_ANCHOR_L=CENTER
    IMAGE_ANCHOR_U=CENTER
    def __init__(self, master, variables, Width = 80, Height = 400, min_val = 0, max_val = 1, padY=3,
                    image_anchorU=CENTER, image_anchorL=CENTER, imageL=None, imageU=None,
                    auto=True, line_width=3, bar_radius=10,
                    bar_color_inner='#5c8a8a', line_s_color="#0a50ff", bar_color_outer='#c2d6d6', line_color = '#476b6b', bgColor='#ffffff',
                    show_value = True, digit_precision='.1f', valueSide='LEFT', font_family='Times', font_size=16, suffix=""):
        RangeSliderV.LINE_COLOR=line_color
        RangeSliderV.LINE_WIDTH=line_width
        RangeSliderV.BAR_COLOR_INNER=bar_color_inner
        RangeSliderV.BAR_COLOR_OUTTER=bar_color_outer
        RangeSliderV.BAR_RADIUS=bar_radius
        RangeSliderV.BAR_RADIUS_INNER=bar_radius-5
        RangeSliderV.DIGIT_PRECISION=digit_precision
        RangeSliderV.FONT_SIZE=font_size
        RangeSliderV.FONT_FAMILY=font_family
        RangeSliderV.IMAGE_ANCHOR_L=image_anchorL
        RangeSliderV.IMAGE_ANCHOR_U=image_anchorU
        RangeSliderV.LINE_S_COLOR=line_s_color
        if auto:
            if imageL!=None or imageU!=None:
                raise Exception("Can't decide if to use auto shape or images!")
            else:
                critical1=max(RangeSliderV.BAR_RADIUS, RangeSliderV.FONT_SIZE*1.33/2)
                critical2=2*(RangeSliderV.BAR_RADIUS+max(len(str(min_val)),len(str(max_val)))*RangeSliderV.FONT_SIZE/1.2)
                critical3=2*(padY+RangeSliderV.BAR_RADIUS)
                if show_value and padY<critical1:
                    raise Exception("padY too small, handle won't be visible completely, as per given condition padY should be atleast "+str(critical1+1)+"px.")
                if Width<critical2:
                    raise Exception("Dimensions incompatible, consider decreasing bar_radius or FONT_SIZE or consider increasing widget width, as per given conditios width should be atleast "+str(critical2+1)+"px.")
                if Height<critical3:
                    raise Exception("Dimensions incompatible, consider decreasing bar_radius or consider increasing widget height, as per given conditios height should be atleast "+str(critical3+1)+"px.")
                if RangeSliderV.BAR_RADIUS<=line_width:
                    raise Exception("bar_radius too small, should be minimum equal to line_width(default=3).")
                self.draw='auto'
        else:
            if imageL==None or imageU==None:
                raise Exception("Handle for one image missing.")
            else:
                critical1=(imageL.height()+padY)*2
                critical2=imageL.width()+max(len(str(min_val)),len(str(max_val)))*RangeSliderV.FONT_SIZE/1.2
                critical3=max(imageU.height()/2, RangeSliderV.FONT_SIZE*1.33/2)
                if imageL.height()==imageU.height() and imageL.width()==imageU.width():
                    if critical1<Height and critical2< Width:
                        if show_value and padY<critical3:
                            raise Exception("padY too small, value won't be visible completely, padY mimumum expected is "+str(critical3)+"px.")
                        self.draw='image'
                    else:
                        raise Exception("Image dimensions not suited with widget Height and width, minimum height expected is "+str(critical1)+"px and minimum width expected is "+str(critical2)+"px.")
                else:
                    raise Exception("Image dimensions incompatible, width and height of both handles should be same respectively.")
        Frame.__init__(self, master, height = Height, width = Width)
        self.pady=padY
        self.ImageL=imageL
        self.ImageU=imageU
        self.master = master
        self.max_val = max_val
        self.min_val = min_val
        self.show_value = show_value
        self.valueSide=valueSide
        self.H = Height
        self.W = Width
        self.canv_H = self.H
        self.canv_W = self.W
        self.suffix=suffix
        self.variables = variables
        try:
            self.init_lis = [variables[0].get(), variables[1].get()]
        except Exception:
            self.init_lis = [min_val, max_val]
        if not show_value:
            self.slider_x = self.canv_W/2 # y pos of the slider
        else:
            if self.valueSide=='LEFT':
                if self.draw=='auto':
                    self.slider_x = self.canv_W-RangeSliderV.BAR_RADIUS-2
                elif self.draw=='image':
                    self.slider_x = self.canv_W-self.ImageL.width()/2-2
            elif self.valueSide=='RIGHT':
                if self.draw=='auto':
                    self.slider_x = RangeSliderV.BAR_RADIUS+2
                else:
                    self.slider_x = self.ImageL.width()/2+2
            else:
                raise Exception("valueSide can either be LEFT or RIGHT")
        self.slider_y=self.pady

        self.bars = []
        self.selected_idx = None # current selection bar index
        for value in self.init_lis:
            pos = (value-min_val)/(max_val-min_val)
            ids = []
            bar = {"Pos":pos, "Ids":ids, "Value":value}
            self.bars.append(bar)


        self.canv = Canvas(self, height = self.canv_H, width = self.canv_W, bg=bgColor ,bd=0, highlightthickness=0, relief='ridge')
        self.canv.pack()
        self.canv.bind("<Motion>", self._mouseMotion)
        self.canv.bind("<B1-Motion>", self._moveBar)
        self.track = self.__addTrack(self.slider_x, self.slider_y, self.slider_x, self.canv_H-self.slider_y, self.bars[0]["Pos"], self.bars[1]["Pos"])
        tempIdx=0
        for bar in self.bars:
            bar["Ids"] = self.__addBar(bar["Pos"],tempIdx)
            tempIdx+=1

        self.__setValues()


    def getValues(self):
        values = [bar["Value"] for bar in self.bars]
        return values

    def __setValues(self):
        values=self.getValues()
        self.variables[0].set(values[0])
        self.variables[1].set(values[1])

    def getPos(self):
        poss = [bar["Pos"] for bar in self.bars]
        return poss

    def _mouseMotion(self, event):
        x = event.x; y = event.y
        selection = self.__checkSelection(x,y)
        if selection[0]:
            self.canv.config(cursor = "hand2")
            self.selected_idx = selection[1]
        else:
            self.canv.config(cursor = "")
            self.selected_idx = None

    def _moveBar(self, event):
        x = event.x; y = event.y
        if self.selected_idx == None:
            return False
        pos = self.__calcPos(y)
        idx = self.selected_idx
        self.__moveBar(idx,pos)

    def __addTrack(self, startx, starty, endx, endy, posL, posU):
        rangeOutL = self.canv.create_line(startx, starty+(1-posL)*(endy-starty), startx, endy, fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        rangeS = self.canv.create_line(startx, starty+(1-posU)*(endy-starty), startx, starty+(1-posL)*(endy-starty), fill = RangeSliderV.LINE_S_COLOR, width = RangeSliderV.LINE_WIDTH)
        rangeOutU = self.canv.create_line(startx, starty, endx, starty+(1-posU)*(endy-starty), fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        return [rangeOutL, rangeS, rangeOutU]

    def __addTrackL(self, startx, starty, endx, endy, posL, posU):
        rangeOutL = self.canv.create_line(startx, starty+(1-posL)*(endy-starty), startx, endy, fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        rangeS = self.canv.create_line(startx, starty+(1-posU)*(endy-starty), startx, starty+(1-posL)*(endy-starty), fill = RangeSliderV.LINE_S_COLOR, width = RangeSliderV.LINE_WIDTH)
        # rangeOutU = self.canv.create_line(startx, starty, endx, starty+(1-posU)*(endy-starty), fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        return [rangeOutL, rangeS]

    def __addTrackR(self, startx, starty, endx, endy, posL, posU):
        # rangeOutL = self.canv.create_line(startx, starty+(1-posL)*(endy-starty), startx, endy, fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        rangeS = self.canv.create_line(startx, starty+(1-posU)*(endy-starty), startx, starty+(1-posL)*(endy-starty), fill = RangeSliderV.LINE_S_COLOR, width = RangeSliderV.LINE_WIDTH)
        rangeOutU = self.canv.create_line(startx, starty, endx, starty+(1-posU)*(endy-starty), fill = RangeSliderV.LINE_COLOR, width = RangeSliderV.LINE_WIDTH)
        return [rangeS, rangeOutU]

    def __addBar(self, pos, tempIdx=None):
        """@ pos: position of the bar, ranged from (0,1)"""
        if pos <0 or pos >1:
            raise Exception("Pos error - Pos: "+str(pos))
        if self.draw=='auto':
            R = RangeSliderV.BAR_RADIUS
            r = RangeSliderV.BAR_RADIUS_INNER
            L = self.canv_H - 2*self.slider_y
            y = self.slider_y+(1-pos)*L
            x = self.slider_x
            id_outer = self.canv.create_oval(x-R,y-R,x+R,y+R, fill = RangeSliderV.BAR_COLOR_OUTTER, width = 2, outline = "")
            id_inner = self.canv.create_oval(x-r,y-r,x+r,y+r, fill = RangeSliderV.BAR_COLOR_INNER, outline = "")
            if self.show_value:
                if self.valueSide=='LEFT':
                    x_value = x-RangeSliderV.BAR_RADIUS-RangeSliderV.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x_value,y,anchor=E, text = format(value, RangeSliderV.DIGIT_PRECISION)+self.suffix,font=(RangeSliderV.FONT_FAMILY,RangeSliderV.FONT_SIZE))
                elif self.valueSide=='RIGHT':
                    x_value = x+RangeSliderV.BAR_RADIUS+RangeSliderV.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x_value,y,anchor=W, text = format(value, RangeSliderV.DIGIT_PRECISION)+self.suffix,font=(RangeSliderV.FONT_FAMILY,RangeSliderV.FONT_SIZE))
                else:
                    raise Exception("valueSide can either be LEFT or RIGHT.")
                return [id_outer, id_inner, id_value]
            else:
                return [id_outer, id_inner]
        elif self.draw=='image':
            L=self.canv_H - 2*self.slider_y
            y=self.slider_y+(1-pos)*L
            x=self.slider_x
            if tempIdx==0:
                imageH=self.canv.create_image(x,y,anchor=RangeSliderV.IMAGE_ANCHOR_L,image=self.ImageL)
            elif tempIdx==1:
                imageH=self.canv.create_image(x,y,anchor=RangeSliderV.IMAGE_ANCHOR_U,image=self.ImageU)
            if self.show_value:
                if self.valueSide=='LEFT':
                    x_value = x-self.ImageL.width()/2-RangeSliderV.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x_value,y, anchor=E, text = format(value, RangeSliderV.DIGIT_PRECISION)+self.suffix,font=(RangeSliderV.FONT_FAMILY,RangeSliderV.FONT_SIZE))
                elif self.valueSide=='RIGHT':
                    x_value = x+self.ImageL.width()/2+RangeSliderV.FONT_SIZE/2
                    value = pos*(self.max_val - self.min_val)+self.min_val
                    id_value = self.canv.create_text(x_value,y, anchor=W, text = format(value, RangeSliderV.DIGIT_PRECISION)+self.suffix,font=(RangeSliderV.FONT_FAMILY,RangeSliderV.FONT_SIZE))
                else:
                    raise Exception("valueSide can either be LEFT or RIGHT")  
                return [imageH, id_value]
            else:
                return [imageH]


    def __moveBar(self, idx, pos):
        positions=self.getPos()
        current_pos=self.bars[idx]["Pos"]
        for i in range(0,2):
            ids=self.bars[i]["Ids"]
            for id in ids:
                self.canv.delete(id)
        for trackComponentsId in self.track[idx:idx+2]:
            self.canv.delete(trackComponentsId)
        if idx==0:
            otherIdx=1
            otherPos=positions[1]
            if pos<=otherPos:
                pos=pos
            else:
                pos=current_pos
            self.track[0:2] = self.__addTrackL(self.slider_x, self.slider_y, self.slider_x, self.canv_H-self.slider_y, pos, otherPos)
        else:
            otherIdx=0
            otherPos=positions[0]
            if pos>=otherPos:
                pos=pos
            else:
                pos=current_pos
            self.track[1:3] = self.__addTrackR(self.slider_x, self.slider_y, self.slider_x, self.canv_H-self.slider_y, otherPos, pos)
        
        self.bars[idx]["Ids"] = self.__addBar(pos, idx)
        self.bars[idx]["Pos"] = pos
        self.bars[idx]["Value"] = pos*(self.max_val - self.min_val)+self.min_val

        self.bars[otherIdx]["Ids"] = self.__addBar(otherPos, otherIdx)
        self.bars[otherIdx]["Pos"] = otherPos
        self.bars[otherIdx]["Value"] = otherPos*(self.max_val - self.min_val)+self.min_val

        self.__setValues()

    def __calcPos(self, y):
        """calculate position from x coordinate"""
        pos = (y - self.slider_y)/(self.canv_H-2*self.slider_y)
        pos = 1-pos
        if pos<0:
            return 0
        elif pos>1:
            return 1
        else:
            return pos

    def __checkSelection(self, x, y):
        """
        To check if the position is inside the bounding rectangle of a Bar
        Return [True, bar_index] or [False, None]
        """
        for idx in range(len(self.bars)):
            id = self.bars[idx]["Ids"][0]
            bbox = self.canv.bbox(id)
            if bbox[0] < x and bbox[2] > x and bbox[1] < y and bbox[3] > y:
                return [True, idx]
        return [False, None]



import tkinter.messagebox as mb
import tkinter.simpledialog as sd

import cv2
import matplotlib.pyplot as plt
import numpy as np
import pymedphys
from matplotlib.backends.backend_agg import FigureCanvasAgg
from matplotlib.figure import Figure
from matplotlib.ticker import AutoMinorLocator
from mpl_toolkits.axes_grid1.inset_locator import inset_axes, mark_inset
from PIL import Image, ImageTk

from ..functions import dp
from ..resources.language import Text
from .egs_import import EGSSimulation
from .measurement_import import Measurement
from .ptw_import import PTWMultimporter
from .sim_import import Simulation


class DoseFigureHandler:
    def __init__(self, parent):

        self.parent = parent

        self.norm = self.parent.norm.get()
        self.calcparams = self.parent.calcparams.get()
        self.normvalue = "max"
        self.errlimval = "absolute"
        self.errlimmin = 1.1
        self.errlimmax = 1.1
        self.errorbars = True
        self.caxcorrection = False
        self.xaxisname = None
        self.diffplot = False
        self.zoom = self.parent.zoom.get()
        self.half = self.parent.half.get()
        self.axlims = (self.parent.axlims.get(), self.parent.axlims.get())
        self.addmenu = False

        # Initialize the current settings
        self.lang = self.parent.lang
        self.text = Text()
        self.style = plt.style.use("default")
        self.colors = self.parent.profile.get_attribute("colors")
        self.marker = self.parent.profile.get_attribute("markers")
        self.markersize = self.parent.profile.get_attribute("markersize")
        self.linewidth = self.parent.profile.get_attribute("linewidth")

        # Initialize necessary variables
        self.props = dict(boxstyle="round", facecolor="wheat", alpha=0.6)
        self.plots = []

        self.difference = []

    def add_plot_data(self, datanames):

        """Adds data to the plot queue
        """
        self.data = []
        test = self.parent.direction
        for tuple in datanames:
            filename = tuple[0]
            type = tuple[1]
            if filename not in [plotdata.filepath for plotdata in self.plots]:
                if type == "simulation" or type == "egs":

                    if type == "simulation":
                        sim = Simulation(filename)
                    elif type == "egs":
                        sim = EGSSimulation(filename)

                    if test == None:
                        self.plots += [sim]
                        self.parent.direction = sim.direction
                        test = self.parent.direction
                    elif test == "Z" and sim.direction == "Z":
                        self.plots += [sim]
                    elif test == "X" and sim.direction == "X":
                        self.plots += [sim]
                    elif test == "Y" and sim.direction == "X":
                        self.plots += [sim]
                    elif test == "X" and sim.direction == "Y":
                        self.plots += [sim]
                    elif test == "Y" and sim.direction == "Y":
                        self.plots += [sim]
                    else:
                        sd.messagebox.showinfo(
                            "", f"{sim.filename}" + self.text.incordata[self.lang][1]
                        )
                        self.parent.filenames.pop(-1)

                elif type == "ptw":
                    importer = PTWMultimporter(filename, self.parent)
                    importer.window.mainloop()
                    importer.window.destroy()
                    measurements = [plot for plot in importer.plots]
                    fails = []
                    for plot in measurements:
                        if test == None:
                            self.plots += [plot]
                            self.parent.direction = plot.direction
                            test = self.parent.direction
                        elif test == "Z" and plot.direction == "Z":
                            self.plots += [plot]
                        elif test == "X" and plot.direction == "X":
                            self.plots += [plot]
                        elif test == "Y" and plot.direction == "X":
                            self.plots += [plot]
                        elif test == "X" and plot.direction == "Y":
                            self.plots += [plot]
                        elif test == "Y" and plot.direction == "Y":
                            self.plots += [plot]
                        else:
                            fails += [plot]

                    if fails != []:
                        self.parent.filenames.pop(-1)
                        names = ""
                        for f in fails:
                            names += str(f.filename) + ", "
                        names = names[:-2]
                        var = 1
                        if len(fails) >= 2:
                            var = 2
                        sd.messagebox.showinfo(
                            "", f"{names}" + self.text.incordata[self.lang][var],
                        )

                    del importer

                else:
                    measurement = Measurement(filename, self.parent)
                    if test == None:
                        self.plots += [measurement]
                        self.parent.direction = measurement.direction
                        test = self.parent.direction
                    elif test == "Z" and measurement.direction == "Z":
                        self.plots += [measurement]
                    elif test == "X" and measurement.direction == "X":
                        self.plots += [measurement]
                    elif test == "Y" and measurement.direction == "X":
                        self.plots += [measurement]
                    elif test == "X" and measurement.direction == "Y":
                        self.plots += [measurement]
                    elif test == "Y" and measurement.direction == "Y":
                        self.plots += [measurement]
                    else:
                        sd.messagebox.showinfo(
                            "",
                            f"{measurement.filename}"
                            + self.text.incordata[self.lang][1],
                        )
                        self.parent.filenames.pop(-1)

        if self.plots[0].direction == "Z":
            self.half = False

        for plotdata in self.plots:
            if self.norm == True:
                if self.normvalue == "max":
                    plotdata.normpoint = max(plotdata.dose[self.half])
                elif self.normvalue == "plateau":
                    plotdata.normpoint = np.average(
                        [
                            plotdata.dose[self.half][
                                len(plotdata.dose[self.half]) // 2
                                - 5 : len(plotdata.dose[self.half]) // 2
                                + 5
                            ]
                        ]
                    )

                else:
                    plotdata.normpoint = plotdata.dose[self.half][
                        len(plotdata.dose) // 2
                    ]

            else:
                plotdata.normpoint = 1

            caxdev = 0
            if self.caxcorrection == True:
                caxdev = dp.calculate_parameters(
                    plotdata.axis[False], plotdata.dose[False], True
                )

            self.data += [
                [
                    np.array([x + caxdev for x in plotdata.axis[self.half]]).tolist(),
                    plotdata.direction,
                    plotdata.dose[self.half] / plotdata.normpoint,
                    np.array(
                        [x / plotdata.normpoint for x in plotdata.std_dev[self.half]]
                    ),
                ]
            ]

        return

    def difference_plot(self):

        if self.plots[0].direction == "Z":
            self.figsize = [10, 5]
        else:
            self.figsize = [10, 6]

        if self.diffplot == True:

            self.figsize[1] *= 1.33333

            gs_kw = dict(height_ratios=[3, 1])
            self.fig, axd = plt.subplot_mosaic(
                [["top"], ["bottom"]],
                gridspec_kw=gs_kw,
                figsize=self.figsize,
                constrained_layout=True,
                sharex=True,
                dpi=600,
            )
            self.ax = axd["top"]
            self.diffax = axd["bottom"]

        else:
            self.fig = Figure(self.figsize, constrained_layout=True, dpi=600)
            self.ax = self.fig.add_subplot(111)

        self.canvas = FigureCanvasAgg(self.fig)
        self.fig.set_canvas(self.canvas)
        self.set_style()

        if self.diffplot == True:

            data = []
            for index, plot_data in enumerate(self.data[:2]):

                data += [[plot_data[0], plot_data[2]]]

            if np.all(np.diff(data[1][0]) > 0) == False:
                interpolated_data = np.flip(
                    np.interp(
                        np.flip(data[0][0]), np.flip(data[1][0]), np.flip(data[1][1])
                    )
                )
            else:
                interpolated_data = np.interp(
                    data[0][0], data[1][0], data[1][1], left=0, right=0
                )

            if self.errlimval == "absolute":
                self.difference = [
                    (interpolated_data[i] - data[0][1][i])  # / (data[0][1][i])
                    for i in range(len(data[0][0]))
                ]
            else:
                self.difference = [
                    100 * (interpolated_data[i] - data[0][1][i]) / (data[0][1][i])
                    for i in range(len(data[0][0]))
                ]

                self.difference[:] = [
                    x if round(x, 1) != -100 else 0 for x in self.difference
                ]
                self.difference[:] = [
                    x if round(x, 1) != 100 else 0 for x in self.difference
                ]

            label = self.text.error[self.lang] + " [abs]"
            if self.errlimval != "absolute":
                label = label[:-6] + " [%]"

            self.diffax.plot(
                data[0][0],
                self.difference,
                color=self.textcolor,
                linewidth=0.6,
                label=label,
            )

            self.errlim = [
                min(self.difference) * self.errlimmin,
                max(self.difference) * self.errlimmax,
            ]
            self.diffax.set_ylim(self.errlim)
            self.diffax.legend(
                loc="upper right",
                framealpha=0.6,
                facecolor="wheat",
                edgecolor="black",
                fancybox=True,
            )
        return

    def set_style(self):

        """Switches the plotting style according to the current theme
        """

        if self.parent.dark.get() == True:
            self.textcolor = "white"
            self.ax.set_facecolor("#363636")
            self.fig.set_facecolor("#363636")
            if self.diffplot == True:
                self.diffax.set_facecolor("#363636")
        else:
            self.textcolor = "black"
            self.ax.set_facecolor("#ffffff")
            self.fig.set_facecolor("#ffffff")
            if self.diffplot == True:
                self.diffax.set_facecolor("#ffffff")

        self.ax.xaxis.label.set_color(self.textcolor)
        self.ax.yaxis.label.set_color(self.textcolor)

        self.ax.tick_params(axis="x", colors=self.textcolor)
        self.ax.tick_params(axis="y", colors=self.textcolor)

        self.ax.spines["left"].set_color(self.textcolor)
        self.ax.spines["bottom"].set_color(self.textcolor)

        plt.rcParams.update(
            {"text.color": self.textcolor, "axes.labelcolor": self.textcolor}
        )

        if self.diffplot == True:
            self.diffax.xaxis.label.set_color(self.textcolor)
            self.diffax.yaxis.label.set_color(self.textcolor)

            self.diffax.tick_params(axis="x", colors=self.textcolor)
            self.diffax.tick_params(axis="y", colors=self.textcolor)

            self.diffax.spines["left"].set_color(self.textcolor)
            self.diffax.spines["bottom"].set_color(self.textcolor)

        return

    def flush(self):

        """Clears the data
        """

        self.data = []

        return

    def fits(self, image):

        """Scales an image to the width of the screen.
        """

        width = self.parent.parent.winfo_screenwidth()
        scale_factor = image.shape[1] / width
        image = cv2.resize(
            image,
            (width, int(image.shape[0] / scale_factor)),
            interpolation=cv2.INTER_AREA,
        )

        return image

    def set_axis(self):

        """Defines the layout and gridlines of the plot 
        """

        self.ax.clear()
        self.ax.grid(True, which="major")
        self.ax.grid(True, which="minor", color="grey", linewidth=0.1)
        self.ax.xaxis.set_minor_locator(AutoMinorLocator())
        self.ax.yaxis.set_minor_locator(AutoMinorLocator())
        self.ax.tick_params(axis="both", which="minor", length=2)

        if self.diffplot == True:
            self.diffax.grid(True, which="major")
            self.diffax.grid(True, which="minor", color="grey", linewidth=0.1)
            self.diffax.xaxis.set_minor_locator(AutoMinorLocator())
            self.diffax.yaxis.set_minor_locator(AutoMinorLocator())
            self.diffax.tick_params(axis="both", which="minor", length=2)

        return

    def add_legend(self):

        """Adds the legend
        """

        self.ax.legend(
            loc="upper right",
            framealpha=0.6,
            facecolor="wheat",
            edgecolor="black",
            fancybox=True,
        )

        return

    def add_descriptors(self):

        """Adds the calculated parameters as descriptors
        """

        temp = self.data
        new_data = []
        for index, data in enumerate(self.data):
            params = self.plots[index].params()
            new_data += [[data, params]]

        self.data = new_data
        if self.plots[0].direction != "Z":

            if self.half == False:
                positionsx = [
                    0.1,
                    0.1,
                    0.901,
                    0.901,
                    0.1,
                ]
                positionsy = [0.98, 0.68, 0.75, 0.45, 0.38]
            else:
                positionsx = [
                    0.7,
                    0.7,
                    0.901,
                    0.901,
                    0.7,
                ]
                positionsy = [0.98, 0.68, 0.68, 0.38, 0.38]

        for index, plot_data in enumerate(self.data):

            if len(plot_data[1]) == 3:

                Q, dQ, zmax = plot_data[1][0], plot_data[1][1], plot_data[1][2]
                textstr = "Q     = {} ± {}\n{} = {} mm".format(Q, dQ, "$z_{max}$", zmax)
                space = 0
                if dQ == 0:
                    space = 0.05
                if len(str(Q)) == 6:
                    space += 0.01
                if len(str(dQ)) == 6:
                    space += 0.01
                self.ax.text(
                    0.795 + space,
                    0.955 - len(self.plots) * 0.0475 - 0.1 * index,
                    textstr,
                    transform=self.ax.transAxes,
                    fontsize=10,
                    verticalalignment="top",
                    bbox=self.props,
                    color=self.colors[index],
                )

            else:
                (
                    HWB,
                    CAXdev,
                    flat_krieger,
                    flat_stddev,
                    S,
                    Lpenumbra,
                    Rpenumbra,
                    Lintegral,
                    Rintegral,
                ) = (
                    plot_data[1][0],
                    plot_data[1][1],
                    plot_data[1][2],
                    plot_data[1][3],
                    plot_data[1][4],
                    plot_data[1][5],
                    plot_data[1][6],
                    plot_data[1][7],
                    plot_data[1][8],
                )

                if self.caxcorrection == True:
                    CAXdev = 0

                textstr = "{} = {} mm\n{} = {} mm\n{} = {}\n{} = {}\n{} = {} mm\n{} = {} mm\n{} = {}\n{} = {}\n{} = {}".format(
                    self.text.fwhm[self.lang],
                    HWB,
                    "$CAX_{dev}$",
                    CAXdev,
                    "$FLAT_{Krieger}$",
                    flat_krieger,
                    "$FLAT_{stddev}$",
                    flat_stddev,
                    "$Penumbra_{L}$",
                    Lpenumbra,
                    "$Penumbra_{R}$",
                    Rpenumbra,
                    "$Integral_{L}$",
                    Lintegral,
                    "$Integral_{R}$",
                    Rintegral,
                    self.text.symmetry[self.lang],
                    S,
                )
                self.ax.text(
                    positionsx[index],
                    positionsy[index],
                    textstr,
                    transform=self.ax.transAxes,
                    fontsize=10,
                    verticalalignment="top",
                    bbox=self.props,
                    horizontalalignment="center",
                    color=self.colors[index],
                )
        self.data = temp
        return

    def set_x_label(self):

        """Names the x-axis according to the plot type
        """

        directions = [plot.direction for plot in self.plots]
        if directions.count(directions[0]) == len(directions):

            self.xlabel = f"{directions[0]}-{self.text.axis[self.lang]} [mm]"
        else:
            self.xlabel = (
                f"X- {self.text.orr[self.lang]} Y-{self.text.axis[self.lang]} [mm]"
            )

        if self.diffplot == True:
            if self.xaxisname != None:
                self.diffax.set_xlabel(self.xaxisname, size=12)
            else:
                self.diffax.set_xlabel(self.xlabel, size=12)
        else:
            if self.xaxisname != None:
                self.ax.set_xlabel(self.xaxisname, size=12)
            else:
                self.ax.set_xlabel(self.xlabel, size=12)

        self.ax.set_ylabel(self.text.normunit[self.lang], size=12)
        if self.norm == False:
            self.ax.set_ylabel(self.plots[0].unit, size=12)

        return

    def create_plots_from_data(self):

        """Creates the plots from the queue
        """

        for index, plot_data in enumerate(self.data):

            if self.errorbars == True:
                try:
                    self.ax.errorbar(
                        x=plot_data[0],
                        y=plot_data[2],
                        yerr=plot_data[3],
                        fmt="none",
                        ecolor=self.colors[index],
                        elinewidth=0.625,
                        capsize=1.25,
                        capthick=0.25,
                        ms=2.5,
                    )
                except ValueError:
                    pass
            self.ax.plot(
                plot_data[0],
                plot_data[2],
                self.marker[index],
                markersize=self.markersize,
                color=self.colors[index],
                linewidth=self.linewidth,
                label=f"{self.plots[index].filename}",
            )

        try:
            self.axins.remove()
        except:
            pass

        self.set_xlims()

        if self.zoom == True:

            self.canvas.draw()
            self.transform = self.ax.transData
            self.inverted_transform = self.ax.transData.inverted()
            width, height = self.fig.canvas.get_width_height()

            extent = max([max(plot.axis[self.half]) for plot in self.plots])
            reference = [
                plot for plot in self.plots if max(plot.axis[self.half]) == extent
            ][-1]

            try:
                self.focuspoint = self.inverted_transform.transform(
                    (
                        width
                        * (
                            -(
                                self.parent.canvas.winfo_width()
                                - self.parent.canvas.image.width()
                            )
                            // 2
                            + self.parent.canvas.coords(self.parent.oval)[0]
                            + 5
                        )
                        / self.parent.canvas.image.width(),
                        1,
                    )
                )
                self.focuspoint = reference.axis[self.half][
                    np.abs(
                        np.asarray(reference.axis[self.half] - self.focuspoint[0])
                    ).argmin()
                ]
            except AttributeError:
                self.focuspoint = reference.axis[self.half][
                    len(reference.axis[self.half]) // 2
                ]
            except IndexError:
                self.focuspoint = reference.axis[self.half][
                    len(reference.axis[self.half]) // 2
                ]

            pixels = self.transform.transform(
                np.vstack([reference.axis[self.half], reference.dose[self.half]]).T
            )
            x, y = pixels.T
            y = height - y

            self.pixelx = x[reference.axis[self.half].index(self.focuspoint)] / width
            self.pixely = y[reference.axis[self.half].index(self.focuspoint)] / height

            loc = "lower left"
            if self.half == False and self.plots[0].direction != "Z":
                if self.ax.get_xlim()[1] > 30:
                    loc = "lower center"

            self.axins = inset_axes(self.ax, "28%", "28%", loc=loc)
            if self.parent.dark.get() == True:
                self.axins.set_facecolor("#363636")

            self.axins.xaxis.label.set_color(self.textcolor)
            self.axins.yaxis.label.set_color(self.textcolor)

            self.axins.tick_params(axis="x", colors=self.textcolor)
            self.axins.tick_params(axis="y", colors=self.textcolor)

            self.axins.spines["left"].set_color(self.textcolor)
            self.axins.spines["bottom"].set_color(self.textcolor)

            yvalsat195 = []
            yvalsat205 = []
            for index, plot_data in enumerate(self.data):
                if self.errorbars == True:
                    try:
                        self.axins.errorbar(
                            x=plot_data[0],
                            y=plot_data[2],
                            yerr=plot_data[3],
                            fmt="none",
                            ecolor=self.colors[index],
                            elinewidth=0.625,
                            capsize=1.25,
                            capthick=0.25,
                            ms=2.5,
                        )
                    except ValueError:
                        pass
                self.axins.plot(
                    plot_data[0],
                    plot_data[2],
                    self.marker[index],
                    markersize=self.markersize,
                    color=self.colors[index],
                    linewidth=self.linewidth,
                    label=f"{self.plots[index].filename}",
                )
                yvalsat195 += [
                    plot_data[2][
                        plot_data[0].index(
                            min(
                                plot_data[0],
                                key=lambda x: abs(x - (self.focuspoint - 5)),
                            )
                        )
                    ]
                ]
                yvalsat205 += [
                    plot_data[2][
                        plot_data[0].index(
                            min(
                                plot_data[0],
                                key=lambda x: abs(x - (self.focuspoint + 5)),
                            )
                        )
                    ]
                ]

            # sub region of the original image
            x1, x2, y1, y2 = (
                self.focuspoint - 5,
                self.focuspoint + 5,
                0.95 * min(min(yvalsat195), min(yvalsat205)),
                1.05 * max(max(yvalsat195), max(yvalsat205)),
            )
            loc1 = 2
            loc2 = 4

            if y2 < y1:
                y1, y2 = y2, y1
                loc1, loc2 = 1, 3

            if self.half == True:
                x1, x2 = x1 + 3, x2 - 3

            self.axins.set_xlim(x1, x2)
            self.axins.set_ylim(y1, y2)
            self.axins.tick_params(
                labelleft=False,
                labelright=True,
                labelbottom=False,
                labeltop=True,
                bottom=False,
                left=False,
            )

            mark_inset(self.ax, self.axins, loc1=loc1, loc2=loc2, fc="none", ec="0.5")

        return

    def set_xlims(self):

        if self.parent.current_limits == []:
            self.current_limits = [self.ax.get_xlim()[0], self.ax.get_xlim()[1]]
            self.parent.current_limits = self.current_limits
            self.parent.initial_limits = self.current_limits
            self.parent.slidervars[0].set(self.current_limits[0])
            self.parent.slidervars[1].set(self.current_limits[1])

        self.current_limits = [
            self.parent.slidervars[0].get(),
            self.parent.slidervars[1].get(),
        ]

        self.ax.set_xlim(self.current_limits)

    def gamma(self):

        gamma = pymedphys.gamma(
            (self.data[0][0],),
            self.data[0][2],
            (self.data[1][0],),
            self.data[1][2],
            self.parent.gammathreshold,
            self.parent.gammadist,
            quiet=True,
        )

        valid_gamma = gamma[~np.isnan(gamma)]
        pass_ratio = np.sum(valid_gamma <= 1) / len(valid_gamma)

        mb.showinfo(
            "Gamma-Index",
            f"ɣ ({self.parent.gammaacc.get()}) = {round(pass_ratio,4)}\n∅ = {round(np.average(valid_gamma),4)}",
        )

    def return_figure(self, filenames):

        """Passes the image of the plot and the plot type to the MainApplication
        """
        try:
            plt.close(self.fig)
        except AttributeError:
            pass

        self.half = self.parent.half.get()
        try:
            self.add_plot_data(filenames)
            self.difference_plot()
        except IndexError:
            self.parent.filenames.pop(-1)
            return
        self.set_axis()
        self.set_x_label()
        self.create_plots_from_data()
        if self.calcparams == True:
            try:
                self.add_descriptors()
            except Exception as e:
                self.parent.calcparams.set(False)
                self.calcparams = False
                sd.messagebox.showwarning("", self.text.calcfail[self.lang])

        self.add_legend()
        self.ax.set_aspect("auto")

        if self.half == True:

            xlim_r = [plots.axis[self.half] for plots in self.plots]
            xlim_r = max([item for sublist in xlim_r for item in sublist])
            self.ax.set_xlim(0, xlim_r)

        self.canvas.draw()
        buffer = self.canvas.buffer_rgba()
        image_array = np.asarray(buffer)
        fit_array = self.fits(image_array)
        image = Image.fromarray(fit_array)
        photoimage = ImageTk.PhotoImage(image)

        return photoimage, self.data[0][1]



import ctypes
import os
import shutil
import site
import sys
import urllib.request


class InstallDnD:
    def __init__(self):

        self.install_success = False
        self.message = ""

        try:
            operating_system = sys.platform

            if "linux" in operating_system:
                operating_system = "linux-x64.tgz"

            if "darwin" in operating_system:
                operating_system = "osx-x64.tgz"

            if "win" in operating_system and "darwin" not in operating_system:
                if ctypes.sizeof(ctypes.c_voidp) == 4:
                    operating_system = "windows-x86.zip"
                else:
                    operating_system = "windows-x64.zip"

            package_paths = site.getsitepackages()
            tcl_path = os.path.join(package_paths[0], "tcl")
            site_path = package_paths[1]
            cur_dir = os.path.dirname(os.path.realpath(__file__))
            os.path.join(cur_dir, "..", "resources")

            tcl_link = (
                "https://github.com/petasis/tkdnd/releases/download/tkdnd-release-test-v2.9.2/tkdnd-2.9.2-"
                + f"{operating_system}"
            )
            urllib.request.urlretrieve(
                tcl_link,
                os.path.join(
                    os.path.join(cur_dir, "..", "resources", "temp"),
                    os.path.basename(tcl_link),
                ),
            )
            shutil.unpack_archive(
                os.path.join(
                    os.path.join(cur_dir, "..", "resources", "temp"),
                    os.path.basename(tcl_link),
                ),
                os.path.realpath(os.path.join(cur_dir, "..", "resources", "temp")),
            )
            os.remove(
                os.path.join(
                    os.path.join(cur_dir, "..", "resources", "temp"),
                    os.path.basename(tcl_link),
                )
            )
            shutil.move(
                os.path.join(
                    os.path.realpath(os.path.join(cur_dir, "..", "resources", "temp")),
                    os.listdir(
                        os.path.realpath(
                            os.path.join(cur_dir, "..", "resources", "temp")
                        )
                    )[0],
                ),
                tcl_path,
            )

            shutil.copytree(
                os.path.join(cur_dir, "..", "resources", "TkinterDnD2"),
                os.path.join(os.path.realpath(site_path), "TkinterDnD2"),
            )

            self.install_success = True
        except Exception as e:
            self.message = e



# dnd actions
PRIVATE = 'private'
NONE = 'none'
ASK = 'ask'
COPY = 'copy'
MOVE = 'move'
LINK = 'link'
REFUSE_DROP = 'refuse_drop'
# dnd types
DND_TEXT = 'DND_Text'
DND_FILES = 'DND_Files'
DND_ALL = '*'
CF_UNICODETEXT = 'CF_UNICODETEXT'
CF_TEXT = 'CF_TEXT'
CF_HDROP = 'CF_HDROP'
FileGroupDescriptor = 'FileGroupDescriptor - FileContents'# ??
FileGroupDescriptorW = 'FileGroupDescriptorW - FileContents'# ??

from TkinterDnD2 import TkinterDnD



import webbrowser
from tkinter import simpledialog as sd

import requests

from .profile import ProfileHandler


class CheckForUpdates:
    def __init__(self):

        currentVersion = "20.0.2"
        try:
            newestVersion = requests.get(
                "https://api.github.com/repos/sebasj13/topasgraphsim/releases/latest"
            ).json()["name"][1:]
        except Exception:
            return

        self.update = {
            "de": f"Eine neue Version ({newestVersion}) von TopasGraphSim ist verfügbar.\nAnsehen?",
            "en": f"A new version ({newestVersion}) of TopasGraphSim is available!\nView?",
        }

        if currentVersion != newestVersion:
            dialog = sd.messagebox.askokcancel(
                "", message=self.update[ProfileHandler().get_attribute("language")]
            )

            if dialog == True:
                try:
                    webbrowser.open("https://pypi.org/project/topasgraphsim/")
                except Exception:
                    return
